# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""services mcp enable command."""

from googlecloudsdk.api_lib.services import services_util
from googlecloudsdk.api_lib.services import serviceusage
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.services import common_flags
from googlecloudsdk.core import log
from googlecloudsdk.core import properties
from googlecloudsdk.core.console import console_io

_OP_BASE_CMD = 'gcloud beta services operations '
_OP_WAIT_CMD = _OP_BASE_CMD + 'wait {0}'

_SERVICE = 'services/%s'
_PROJECT_RESOURCE = 'projects/{}'
_FOLDER_RESOURCE = 'folders/{}'
_ORGANIZATION_RESOURCE = 'organizations/{}'
_CONSUMER_POLICY_DEFAULT = '/consumerPolicies/{}'


@base.UniverseCompatible
@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA)
class Enable(base.SilentCommand):
  """Enable a service for MCP on a project, folder or organization.

  Enable a service for MCP on a project, folder or organization

  ## EXAMPLES

  To enable a service for MCP called `my-service` on the current project, run:

    $ {command} my-service

  To enable a service for MCP called `my-service` on the project
  `my-project`, run:

    $ {command} my-service --project=my-project

  To enable a service for MCP called `my-service` on the folder
  `my-folder, run:

    $ {command} my-service --folder=my-folder

  To enable a service for MCP called `my-service` on the organization
  `my-organization`, run:

    $ {command} my-service --organization=my-organization

  To run the same command asynchronously (non-blocking), run:

    $ {command} my-service --async
  """

  @staticmethod
  def Args(parser):
    """Args is called by calliope to gather arguments for this command.

    Args:
      parser: An argparse parser that you can use to add arguments that go on
        the command line after this command. Positional arguments are allowed.
    """
    common_flags.service_flag(suffix='to enable MCP').AddToParser(parser)
    common_flags.add_resource_args(parser)
    common_flags.skip_mcp_endpoint_check_flag(parser)

    base.ASYNC_FLAG.AddToParser(parser)

  def Run(self, args):
    """Run 'services mcp enable'.

    Args:
      args: argparse.Namespace, The arguments that this command was invoked
        with.

    Returns:
      Updated MCP Policy.
    """
    project = properties.VALUES.core.project.Get(required=True)
    resource_name = _PROJECT_RESOURCE.format(project)
    if args.IsSpecified('project'):
      resource_name = _PROJECT_RESOURCE.format(args.project)
      project = args.project
    if args.IsSpecified('folder'):
      resource_name = _FOLDER_RESOURCE.format(args.folder)
      folder = args.folder
    else:
      folder = None
    if args.IsSpecified('organization'):
      resource_name = _ORGANIZATION_RESOURCE.format(args.organization)
      organization = args.organization
    else:
      organization = None

    # check if sevice has Mcp Config
    service_metadata = serviceusage.GetServiceV2Beta(
        f'{resource_name}/services/{args.service}'
    )
    if not args.skip_mcp_endpoint_check and (
        not service_metadata.service.mcpServer
        or not service_metadata.service.mcpServer.urls
    ):
      log.error(
          f'The service {args.service} does not have MCP endpoint.'
      )
      return

    log.warning(
        'MCP enablement is not required and only service enablement will be'
        ' performed.'
    )
    if not service_metadata.state.enableRules:
      enable_msg = serviceusage.GetMcpEnabledError(
          service=args.service, resource_name=resource_name
      )
      do_enable = console_io.PromptContinue(
          enable_msg,
          default=False,
          throw_if_unattended=True,
      )
      if do_enable:
        enable_service_op = serviceusage.AddEnableRule(
            [args.service],
            project,
            folder=folder,
            organization=organization,
        )

        if args.async_:
          cmd = _OP_WAIT_CMD.format(enable_service_op.name)
          log.status.Print(
              'Asynchronous operation is in progress... '
              'Use the following command to wait for its '
              f'completion:\n {cmd}'
          )
          return

        # The operation should not be None when enable rules are empty,
        # but in case it is, we check it here to avoid error.
        if enable_service_op:
          enable_service_op = services_util.WaitOperation(
              enable_service_op.name, serviceusage.GetOperationV2Beta
          )

          if enable_service_op.error:
            log.error(
                f'Failed to enable the service {args.service} for the resource'
                f' {resource_name}: {enable_service_op.error}'
            )
            return
          else:
            log.status.Print(
                f'The service {args.service} has been enabled for the'
                f' resource {resource_name}.'
            )
            return
      else:
        return

    else:
      log.status.Print(
          f'The service {args.service} has'
          f' been enabled for the resource {resource_name}.'
      )
      return



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Implementation of cancel command for batch operations jobs."""


from googlecloudsdk.api_lib.storage import storage_batch_operations_api
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.storage.batch_operations.jobs import resource_args
from googlecloudsdk.core import log


@base.DefaultUniverseOnly
class Cancel(base.Command):
  """Cancel a batch-operations job."""

  hints = base.CommandHint(read_only=False)

  detailed_help = {
      "DESCRIPTION": """
      Cancel the batch operation job.
      """,
      "EXAMPLES": """
      To cancel a batch job with the name `my-job` in location `us-central1`:

          $ {command} my-job --location=us-central1

      To cancel the same batch job with fully specified name:

          $ {command} projects/my-project/locations/us-central1/jobs/my-job
      """,
  }

  @staticmethod
  def Args(parser):
    resource_args.add_batch_job_resource_arg(parser, "to cancel")

  def Run(self, args):
    job_name = args.CONCEPTS.batch_job.Parse().RelativeName()
    storage_batch_operations_api.StorageBatchOperationsApi().cancel_batch_job(
        job_name
    )
    log.status.Print("Canceled batch job: {}".format(job_name))



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Implementation of update command for updating bucket settings."""


from googlecloudsdk.api_lib.storage import cloud_api
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.storage import errors_util
from googlecloudsdk.command_lib.storage import flags
from googlecloudsdk.command_lib.storage import stdin_iterator
from googlecloudsdk.command_lib.storage import storage_url
from googlecloudsdk.command_lib.storage import user_request_args_factory
from googlecloudsdk.command_lib.storage import wildcard_iterator
from googlecloudsdk.command_lib.storage.tasks import task_executor
from googlecloudsdk.command_lib.storage.tasks import task_graph_executor
from googlecloudsdk.command_lib.storage.tasks import task_status
from googlecloudsdk.command_lib.storage.tasks.buckets import update_bucket_task


_CORS_HELP_TEXT = """
Sets the Cross-Origin Resource Sharing (CORS) configuration on a bucket.
An example CORS JSON document looks like the following:

  [
    {
      "origin": ["http://origin1.example.com"],
      "responseHeader": ["Content-Type"],
      "method": ["GET"],
      "maxAgeSeconds": 3600
    }
  ]

For more information about supported endpoints for CORS, see
[Cloud Storage CORS support](https://cloud.google.com/storage/docs/cross-origin#server-side-support).
"""
_LABELS_HELP_TEXT = """
Sets the label configuration for the bucket. An example label JSON document
looks like the following:

  {
    "your_label_key": "your_label_value",
    "your_other_label_key": "your_other_label_value"
  }
"""
_LIFECYCLE_HELP_TEXT = """
Sets the lifecycle management configuration on a bucket. For example,
The following lifecycle management configuration JSON document
specifies that all objects in this bucket that are more than 365 days
old are deleted automatically:

  {
    "rule":
    [
      {
        "action": {"type": "Delete"},
        "condition": {"age": 365}
      }
    ]
  }
"""


def _add_common_args(parser):
  """Register flags for this command.

  Args:
    parser (argparse.ArgumentParser): The parser to add the arguments to.

  Returns:
    buckets update flag group
  """
  parser.add_argument(
      'url',
      nargs='*',
      type=str,
      help='Specifies the URLs of the buckets to update.',
  )
  acl_flags_group = parser.add_group()
  flags.add_acl_modifier_flags(acl_flags_group)

  default_acl_flags_group = parser.add_group()
  default_acl_flags_group.add_argument(
      '--default-object-acl-file',
      help='Sets the default object ACL from file for the bucket.',
  )
  default_acl_flags_group.add_argument(
      '--predefined-default-object-acl',
      help='Apply a predefined set of default object access controls tobuckets',
  )
  default_acl_flags_group.add_argument(
      '--add-default-object-acl-grant',
      action='append',
      metavar='DEFAULT_OBJECT_ACL_GRANT',
      type=arg_parsers.ArgDict(),
      help=(
          'Adds default object ACL grant. See --add-acl-grant help text for'
          ' more details.'
      ),
  )
  default_acl_flags_group.add_argument(
      '--remove-default-object-acl-grant',
      action='append',
      help=(
          'Removes default object ACL grant. See --remove-acl-grant help text'
          ' for more details.'
      ),
  )

  cors = parser.add_mutually_exclusive_group()
  cors.add_argument('--cors-file', help=_CORS_HELP_TEXT)
  cors.add_argument(
      '--clear-cors',
      action='store_true',
      help="Clears the bucket's CORS settings.")
  parser.add_argument(
      '--default-storage-class',
      help='Sets the default storage class for the bucket.',
  )
  default_encryption_key = parser.add_mutually_exclusive_group()
  default_encryption_key.add_argument(
      '--default-encryption-key',
      help='Set the default KMS key for the bucket.')
  default_encryption_key.add_argument(
      '--clear-default-encryption-key',
      action='store_true',
      help="Clears the bucket's default encryption key.")
  parser.add_argument(
      '--default-event-based-hold',
      action=arg_parsers.StoreTrueFalseAction,
      help='Sets the default value for an event-based hold on the bucket.'
      ' By setting the default event-based hold on a bucket, newly-created'
      ' objects inherit that value as their event-based hold (it is not'
      ' applied retroactively).')
  labels = parser.add_mutually_exclusive_group()
  labels.add_argument('--labels-file', help=_LABELS_HELP_TEXT)
  update_labels = labels.add_group()
  update_labels.add_argument(
      '--update-labels',
      metavar='LABEL_KEYS_AND_VALUES',
      type=arg_parsers.ArgDict(),
      help='Add or update labels. Example:'
      ' --update-labels=key1=value1,key2=value2')
  update_labels.add_argument(
      '--remove-labels',
      metavar='LABEL_KEYS',
      type=arg_parsers.ArgList(),
      help='Remove labels by their key names.')
  labels.add_argument(
      '--clear-labels',
      action='store_true',
      help='Clear all labels associated with a bucket.')
  lifecycle = parser.add_mutually_exclusive_group()
  lifecycle.add_argument('--lifecycle-file', help=_LIFECYCLE_HELP_TEXT)
  lifecycle.add_argument(
      '--clear-lifecycle',
      action='store_true',
      help='Removes all lifecycle configuration for the bucket.')
  log_bucket = parser.add_mutually_exclusive_group()
  log_bucket.add_argument(
      '--log-bucket',
      help='Enables usage and storage logging for the bucket specified in the'
      ' overall update command, outputting log files to the bucket specified in'
      ' this flag. Cloud Storage does not validate the existence of the bucket'
      ' receiving logs. In addition to enabling logging on your bucket, you'
      ' also need to grant cloud-storage-analytics@google.com write access to'
      ' the log bucket.')
  log_bucket.add_argument(
      '--clear-log-bucket',
      action='store_true',
      help='Disables usage and storage logging for the bucket specified in the'
      ' overall update command.')
  log_object_prefix = parser.add_mutually_exclusive_group()
  log_object_prefix.add_argument(
      '--log-object-prefix',
      help='Specifies a prefix for the names of logs generated in the log'
      ' bucket. The default prefix is the bucket name. If logging is not'
      ' enabled, this flag has no effect.')
  log_object_prefix.add_argument(
      '--clear-log-object-prefix',
      action='store_true',
      help='Clears the prefix used to determine the naming of log objects in'
      ' the logging bucket.')
  public_access_prevention = parser.add_mutually_exclusive_group()
  public_access_prevention.add_argument(
      '--public-access-prevention',
      '--pap',
      action=arg_parsers.StoreTrueFalseAction,
      help='If True, sets [public access prevention](https://cloud.google.com'
      '/storage/docs/public-access-prevention) to "enforced".'
      ' If False, sets public access prevention to "inherited".')
  public_access_prevention.add_argument(
      '--clear-public-access-prevention',
      '--clear-pap',
      action='store_true',
      help='Unsets the public access prevention setting on a bucket.',
  )
  retention_period = parser.add_mutually_exclusive_group()
  retention_period.add_argument(
      '--retention-period',
      help='Minimum [retention period](https://cloud.google.com'
      '/storage/docs/bucket-lock#retention-periods)'
      ' for objects stored in the bucket, for example'
      ' ``--retention-period=P1Y1M1DT5S\'\'. Objects added to the bucket'
      ' cannot be deleted until they\'ve been stored for the specified'
      ' length of time. Default is no retention period. Only available'
      ' for Cloud Storage using the JSON API.')
  retention_period.add_argument(
      '--clear-retention-period',
      action='store_true',
      help='Clears the object retention period for a bucket.')
  parser.add_argument(
      '--lock-retention-period',
      action='store_true',
      help='Locks an unlocked retention policy on the buckets. Caution: A'
      ' locked retention policy cannot be removed from a bucket or reduced in'
      ' duration. Once locked, deleting the bucket is the only way to'
      ' "remove" a retention policy.')
  parser.add_argument(
      '--requester-pays',
      action=arg_parsers.StoreTrueFalseAction,
      help='Allows you to configure a Cloud Storage bucket so that the'
      ' requester pays all costs related to accessing the bucket and its'
      ' objects.')
  parser.add_argument(
      '--soft-delete-duration',
      type=arg_parsers.Duration(),
      help=(
          'Duration to retain soft-deleted objects. For example, "2w1d" is'
          ' two weeks and one day.'
      ),
  )
  parser.add_argument(
      '--clear-soft-delete',
      action='store_true',
      help=(
          'Clears bucket soft delete settings. Does not affect objects already'
          ' in soft-deleted state.'
      ),
  )
  parser.add_argument(
      '--uniform-bucket-level-access',
      action=arg_parsers.StoreTrueFalseAction,
      help=(
          'Enables or disables [uniform bucket-level access]'
          '(https://cloud.google.com/storage/docs/bucket-policy-only)'
          ' for the buckets.'
      ),
  )
  parser.add_argument(
      '--versioning',
      action=arg_parsers.StoreTrueFalseAction,
      help=(
          'Allows you to configure a Cloud Storage bucket to keep old'
          ' versions of objects.'
      ),
  )
  web_main_page_suffix = parser.add_mutually_exclusive_group()
  web_main_page_suffix.add_argument(
      '--web-main-page-suffix',
      help=(
          'Cloud Storage allows you to configure a bucket to behave like a'
          ' static website. A subsequent GET bucket request through a custom'
          ' domain serves the specified "main" page instead of performing the'
          ' usual bucket listing.'
      ),
  )
  web_main_page_suffix.add_argument(
      '--clear-web-main-page-suffix',
      action='store_true',
      help='Clear website main page suffix if bucket is hosting website.',
  )
  web_error_page = parser.add_mutually_exclusive_group()
  web_error_page.add_argument(
      '--web-error-page',
      help=(
          'Cloud Storage allows you to configure a bucket to behave like a'
          ' static website. A subsequent GET bucket request through a custom'
          ' domain for a non-existent object serves the specified error page'
          ' instead of the standard Cloud Storage error.'
      ),
  )
  web_error_page.add_argument(
      '--clear-web-error-page',
      action='store_true',
      help='Clear website error page if bucket is hosting website.',
  )
  flags.add_additional_headers_flag(parser)
  flags.add_autoclass_flags(parser)
  flags.add_continue_on_error_flag(parser)
  flags.add_recovery_point_objective_flag(parser)
  flags.add_read_paths_from_stdin_flag(parser)
  flags.add_encryption_enforcement_file_flag(parser)

  ip_filter = parser.add_mutually_exclusive_group()
  ip_filter.add_argument(
      '--clear-ip-filter',
      action='store_true',
      help='Disables and clears IP filter configuration of the bucket.',
  )
  flags.add_ip_filter_file_flag(ip_filter)


def _add_alpha_args(parser):
  """Register flags for the alpha version of this command.

  Args:
    parser (argparse.ArgumentParser): The parser to add the arguments to.

  """
  # All alpha flags have been promoted. Hence a no-op.
  del parser


def _is_initial_bucket_metadata_needed(user_request_args):
  """Determines if the bucket update has to patch existing metadata."""
  resource_args = user_request_args.resource_args
  if not resource_args:
    return False
  return user_request_args_factory.adds_or_removes_acls(
      user_request_args) or any([
          resource_args.labels_file_path,
          resource_args.labels_to_append,
          resource_args.labels_to_remove,
      ])


@base.ReleaseTracks(base.ReleaseTrack.GA)
@base.UniverseCompatible
class Update(base.Command):
  """Update bucket settings."""

  hints = base.CommandHint(read_only=False)

  detailed_help = {
      'DESCRIPTION':
          """
      Update the settings for a bucket.
      """,
      'EXAMPLES':
          """

      The following command updates the default storage class of a Cloud Storage
      bucket named "my-bucket" to NEARLINE and sets requester pays to true:

        $ {command} gs://my-bucket --default-storage-class=NEARLINE --requester-pays

      The following command updates the retention period of a Cloud Storage
      bucket named "my-bucket" to one year and thirty-six minutes:

        $ {command} gs://my-bucket --retention-period=1y36m

      The following command clears the retention period of a bucket:

        $ {command} gs://my-bucket --clear-retention-period
      """,
  }

  @staticmethod
  def Args(parser):
    _add_common_args(parser)

  def update_task_iterator(self, args):
    user_request_args = (
        user_request_args_factory.get_user_request_args_from_command_args(
            args, metadata_type=user_request_args_factory.MetadataType.BUCKET
        )
    )
    if user_request_args_factory.adds_or_removes_acls(user_request_args):
      fields_scope = cloud_api.FieldsScope.FULL
    else:
      fields_scope = cloud_api.FieldsScope.NO_ACL

    urls = stdin_iterator.get_urls_iterable(
        args.url, args.read_paths_from_stdin
    )
    for url_string in urls:
      url = storage_url.storage_url_from_string(url_string)
      errors_util.raise_error_if_not_bucket(args.command_path, url)
      for resource in wildcard_iterator.get_wildcard_iterator(
          url_string,
          fields_scope=fields_scope,
          get_bucket_metadata=_is_initial_bucket_metadata_needed(
              user_request_args)):
        yield update_bucket_task.UpdateBucketTask(
            resource, user_request_args=user_request_args)

  def Run(self, args):
    task_status_queue = task_graph_executor.multiprocessing_context.Queue()

    locks_retention_period = getattr(args, 'lock_retention_period', False)

    self.exit_code = task_executor.execute_tasks(
        self.update_task_iterator(args),
        parallelizable=not locks_retention_period,
        task_status_queue=task_status_queue,
        progress_manager_args=task_status.ProgressManagerArgs(
            increment_type=task_status.IncrementType.INTEGER,
            manifest_path=None),
        continue_on_error=args.continue_on_error,
    )


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class UpdateAlpha(Update):
  """Update bucket settings."""

  detailed_help = {
      'DESCRIPTION':
          """
      Update a bucket.
      """,
      'EXAMPLES':
          """

      The following command updates the retention period of a Cloud Storage
      bucket named "my-bucket" to one year and thirty-six minutes:

        $ {command} gs://my-bucket --retention-period=1y36m

      The following command clears the retention period of a bucket:

        $ {command} gs://my-bucket --clear-retention-period
      """,
  }

  @staticmethod
  def Args(parser):
    _add_common_args(parser)
    _add_alpha_args(parser)



# -*- coding: utf-8 -*- #
# Copyright 2016 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""bigtable instances describe command."""


import textwrap

from googlecloudsdk.api_lib.bigtable import util
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.bigtable import arguments
from googlecloudsdk.core import properties
from googlecloudsdk.core import resources


class DescribeInstance(base.DescribeCommand):
  """Describe an existing Bigtable instance."""

  detailed_help = {
      'EXAMPLES':
          textwrap.dedent("""\
          To view an instance's description, run:

            $ {command} my-instance-id

          """),
  }

  @staticmethod
  def Args(parser):
    """Register flags for this command."""
    arguments.AddInstanceResourceArg(parser, 'to describe', positional=True)

  def Run(self, args):
    """This is what gets called when the user runs this command.

    Args:
      args: an argparse namespace. All the arguments that were provided to this
        command invocation.

    Returns:
      Some value that we want to have printed later.
    """
    cli = util.GetAdminClient()
    ref = resources.REGISTRY.Parse(
        args.instance,
        params={
            'projectsId': properties.VALUES.core.project.GetOrFail,
        },
        collection='bigtableadmin.projects.instances')
    msg = util.GetAdminMessages().BigtableadminProjectsInstancesGetRequest(
        name=ref.RelativeName())
    instance = cli.projects_instances.Get(msg)
    return instance



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for retrieving the status of asynchronous replication for a disk-pair."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute import flags
from googlecloudsdk.command_lib.compute.disks import flags as disks_flags

DETAILED_HELP = {
    'brief': (
        'Retrieves the status of asynchronous replication for a Compute'
        ' Engine persistent disk-pair'
    ),
    'DESCRIPTION': """\
        *{command}* fetches the current status of async replication on a Compute
        Engine persistent disk-pair. This command can be invoked on either the
        primary disk or the secondary-disk but the scope respective to the disk
        must be provided.
        """,
    'EXAMPLES': """\
        Replication status can be fetched by using either the primary or the
        secondary disk. To get the current replication status of the disk-pair
        with the primary disk 'primary-disk-1' in zone 'us-east1-a', and project
        'my-project1' and the secondary disk 'secondary-disk-1' in zone
        'us-west1-a', and the project 'my-project2', the following commands can
        be used:

          $ {command} primary-disk-1 --zone=us-east1-a --project=my-project1
          or
          $ {command} secondary-disk-1 --zone=us-west1-a --project=my-project2
        """,
}


def _CommonArgs(parser):
  """Add arguments used for parsing in all command tracks."""
  GetAsyncReplicationStatus.disks_arg.AddArgument(parser)


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
@base.DefaultUniverseOnly
class GetAsyncReplicationStatus(base.Command):
  """Get Async Replication Status for Compute Engine persistent disk-pairs in an asynchronous replication ."""

  @classmethod
  def Args(cls, parser):
    GetAsyncReplicationStatus.disks_arg = disks_flags.MakeDiskArg(plural=False)
    _CommonArgs(parser)

  @classmethod
  def _GetApiHolder(cls, no_http=False):
    return base_classes.ComputeApiHolder(cls.ReleaseTrack(), no_http)

  def Run(self, args):
    return self._Run(args)

  def _Run(self, args):
    compute_holder = self._GetApiHolder()
    client = compute_holder.client

    disk_ref = GetAsyncReplicationStatus.disks_arg.ResolveAsResource(
        args,
        compute_holder.resources,
        scope_lister=flags.GetDefaultScopeLister(client),
    )

    request = None
    if disk_ref.Collection() == 'compute.disks':
      request = client.messages.ComputeDisksGetAsyncReplicationStatusRequest(
          disk=disk_ref.Name(),
          project=disk_ref.project,
          zone=disk_ref.zone,
      )
      request = (
          client.apitools_client.disks,
          'GetAsyncReplicationStatus',
          request,
      )
    elif disk_ref.Collection() == 'compute.regionDisks':
      request = (
          client.messages.ComputeRegionDisksGetAsyncReplicationStatusRequest(
              disk=disk_ref.Name(),
              project=disk_ref.project,
              region=disk_ref.region,
          )
      )
      request = (
          client.apitools_client.regionDisks,
          'GetAsyncReplicationStatus',
          request,
      )
    return client.MakeRequests([request])


GetAsyncReplicationStatus.detailed_help = DETAILED_HELP



# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Wrapper for user-visible error exceptions to raise in the CLI."""


from googlecloudsdk.core import exceptions


class Error(exceptions.Error):
  """Exceptions for Deployment Manager errors."""


class ArgumentError(Error):
  """Command argument error."""


class ConfigError(Error):
  """Config file read, parse or content error."""


class ManifestError(Error):
  """Manfest read, parse or content error."""


class OperationError(Error):
  """Operation encountered an error."""


class OperationTimeoutError(OperationError):
  """Operation timed out."""



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Commands for interacting with the Cloud NetApp Files Volume Replication API resource."""


from apitools.base.py import list_pager
from googlecloudsdk.api_lib.netapp import constants
from googlecloudsdk.api_lib.netapp import util as netapp_api_util
from googlecloudsdk.api_lib.util import waiter
from googlecloudsdk.calliope import base
from googlecloudsdk.core import log
from googlecloudsdk.core import resources


class ReplicationsClient(object):
  """Wrapper for working with Replications in the Cloud NetApp Files API Client."""

  def __init__(self, release_track=base.ReleaseTrack.BETA):
    if release_track == base.ReleaseTrack.BETA:
      self._adapter = BetaReplicationsAdapter()
    elif release_track == base.ReleaseTrack.ALPHA:
      self._adapter = AlphaReplicationsAdapter()
    elif release_track == base.ReleaseTrack.GA:
      self._adapter = ReplicationsAdapter()
    else:
      raise ValueError(
          '[{}] is not a valid API version.'.format(
              netapp_api_util.VERSION_MAP[release_track]
          )
      )

  @property
  def client(self):
    return self._adapter.client

  @property
  def messages(self):
    return self._adapter.messages

  def WaitForOperation(self, operation_ref):
    """Wait on the long-running operation until the done field is True.

    Args:
      operation_ref: the operation reference.

    Raises:
      waiter.OperationError: if the operation contains an error.

    Returns:
      the 'response' field of the Operation.
    """
    return waiter.WaitFor(
        waiter.CloudOperationPollerNoResources(
            self.client.projects_locations_operations
        ),
        operation_ref,
        'Waiting for [{0}] to finish'.format(operation_ref.Name()),
    )

  def CreateReplication(self, replication_ref, volume_ref, async_, config):
    """Creates a Cloud NetApp Volume Replication."""
    request = (
        self.messages.NetappProjectsLocationsVolumesReplicationsCreateRequest(
            parent=volume_ref,
            replicationId=replication_ref.Name(),
            replication=config,
        )
    )
    create_op = self.client.projects_locations_volumes_replications.Create(
        request
    )
    if async_:
      return create_op
    operation_ref = resources.REGISTRY.ParseRelativeName(
        create_op.name, collection=constants.OPERATIONS_COLLECTION
    )
    return self.WaitForOperation(operation_ref)

  def ParseReplicationConfig(
      self,
      name=None,
      description=None,
      labels=None,
      replication_schedule=None,
      destination_volume_parameters=None,
      cluster_location=None,
  ):
    """Parse the command line arguments for Create Replication into a config.

    Args:
      name: the name of the Replication.
      description: the description of the Replication.
      labels: the parsed labels value.
      replication_schedule: the schedule for Replication.
      destination_volume_parameters: the input parameters used for creating
        destination volume.
      cluster_location: location of the user cluster, optional

    Returns:
      the configuration that will be used as the request body for creating a
      Cloud NetApp Files Replication.
    """
    replication = self.messages.Replication()
    replication.name = name
    replication.description = description
    replication.labels = labels
    replication.replicationSchedule = replication_schedule
    self._adapter.ParseDestinationVolumeParameters(
        replication, destination_volume_parameters
    )
    replication.clusterLocation = cluster_location
    return replication

  def ListReplications(self, volume_ref, limit=None):
    """List all active Cloud NetApp Volume Replications.

    Args:
      volume_ref: The parent Volume to list NetApp Volume Replications.
      limit: The number of Cloud NetApp Volume Replications to limit the results
        to. This limit is passed to the server and the server does the limiting.

    Returns:
      Generator that yields the Cloud NetApp Volume Replications.
    """
    request = (
        self.messages.NetappProjectsLocationsVolumesReplicationsListRequest(
            parent=volume_ref
        )
    )
    # Check for unreachable locations.
    response = self.client.projects_locations_volumes_replications.List(request)
    for location in response.unreachable:
      log.warning('Location {} may be unreachable.'.format(location))
    return list_pager.YieldFromList(
        self.client.projects_locations_volumes_replications,
        request,
        field=constants.REPLICATION_RESOURCE,
        limit=limit,
        batch_size_attribute='pageSize',
    )

  def DeleteReplication(self, replication_ref, async_):
    """Delete an existing Cloud NetApp Volume Replication."""
    request = (
        self.messages.NetappProjectsLocationsVolumesReplicationsDeleteRequest(
            name=replication_ref.RelativeName()
        )
    )
    return self._DeleteReplication(async_, request)

  def _DeleteReplication(self, async_, request):
    delete_op = self.client.projects_locations_volumes_replications.Delete(
        request
    )
    if async_:
      return delete_op
    operation_ref = resources.REGISTRY.ParseRelativeName(
        delete_op.name, collection=constants.OPERATIONS_COLLECTION
    )
    return self.WaitForOperation(operation_ref)

  def GetReplication(self, replication_ref):
    """Get information of a Cloud NetApp Volume Replication."""
    request = (
        self.messages.NetappProjectsLocationsVolumesReplicationsGetRequest(
            name=replication_ref.RelativeName()
        )
    )
    return self.client.projects_locations_volumes_replications.Get(request)

  def ParseUpdatedReplicationConfig(
      self, replication_config, description=None, labels=None,
      replication_schedule=None, cluster_location=None
  ):
    """Parse updates into a replication config.

    Args:
      replication_config: The replication config to update.
      description: The new description, if any.
      labels: LabelsValue message, the new labels value, if any.
      replication_schedule: The new schedule for Replication, if any.
      cluster_location: location of the user cluster, if any.

    Returns:
      The replication message.
    """
    return self._adapter.ParseUpdatedReplicationConfig(
        replication_config, description=description, labels=labels,
        replication_schedule=replication_schedule,
        cluster_location=cluster_location,
    )

  def UpdateReplication(
      self, replication_ref, replication_config, update_mask, async_
  ):
    """Update a Cloud NetApp Volume Replication.

    Args:
      replication_ref: the reference to the Replication.
      replication_config: Replication config, the updated replication.
      update_mask: str, a comma-separated list of updated fields.
      async_: bool, if False, wait for the operation to complete.

    Returns:
      an Operation or Volume message.
    """
    update_op = self._adapter.UpdateReplication(
        replication_ref, replication_config, update_mask
    )
    if async_:
      return update_op
    operation_ref = resources.REGISTRY.ParseRelativeName(
        update_op.name, collection=constants.OPERATIONS_COLLECTION
    )
    return self.WaitForOperation(operation_ref)

  def ResumeReplication(
      self, replication_ref, async_
  ):
    """Resume a Cloud NetApp Volume Replication.

    Args:
      replication_ref: the reference to the Replication.
      async_: bool, if False, wait for the operation to complete.

    Returns:
      an Operation or Volume message.
    """
    resume_op = self._adapter.ResumeReplication(replication_ref)
    if async_:
      return resume_op
    operation_ref = resources.REGISTRY.ParseRelativeName(
        resume_op.name, collection=constants.OPERATIONS_COLLECTION
    )
    return self.WaitForOperation(operation_ref)

  def ReverseReplication(
      self, replication_ref, async_
  ):
    """Reverse the direction of a Cloud NetApp Volume Replication.

    Args:
      replication_ref: the reference to the Replication.
      async_: bool, if False, wait for the operation to complete.

    Returns:
      an Operation if async_ is set to true, or a Replication message if the
      ReverseReplicationDirectionRequest is successful.
    """
    reverse_op = self._adapter.ReverseReplication(replication_ref)
    if async_:
      return reverse_op
    operation_ref = resources.REGISTRY.ParseRelativeName(
        reverse_op.name, collection=constants.OPERATIONS_COLLECTION
    )
    return self.WaitForOperation(operation_ref)

  def StopReplication(self, replication_ref, async_, force):
    """Stop a Cloud NetApp Volume Replication.

    Args:
      replication_ref: the reference to the Replication.
      async_: bool, if False, wait for the operation to complete.
      force: bool, if True, call stop Replication with force parameter set to
        True.

    Returns:
      an Operation or Volume message.
    """
    stop_op = self._adapter.StopReplication(replication_ref, force)
    if async_:
      return stop_op
    operation_ref = resources.REGISTRY.ParseRelativeName(
        stop_op.name, collection=constants.OPERATIONS_COLLECTION
    )
    return self.WaitForOperation(operation_ref)

  def ParseEstablishPeeringRequestConfig(
      self,
      peer_cluster_name,
      peer_svm_name,
      peer_volume_name,
      peer_ip_addresses=None,
  ):
    """Parse the command line arguments for Establish Peering into a config.

    Args:
      peer_cluster_name: the name of the peer cluster.
      peer_svm_name: the name of the peer svm.
      peer_volume_name: the name of the peer volume.
      peer_ip_addresses: the ip addresses of the peer cluster.

    Returns:
      the configuration that will be used as the request body for establishing
      peering for Hybrid Replication.
    """
    return self._adapter.ParseEstablishPeeringRequestConfig(
        peer_cluster_name, peer_svm_name, peer_volume_name, peer_ip_addresses
    )

  def EstablishPeering(
      self, replication_ref, establish_peering_request_config, async_
  ):
    """Establish a peering for a Cloud NetApp Volume Replication.

    Args:
      replication_ref: the reference to the Replication.
      establish_peering_request_config: the request configuration to establish
        peering.
      async_: bool, if False, wait for the operation to complete.

    Returns:
      an Operation or Volume message.
    """
    establish_op = self._adapter.EstablishPeering(
        replication_ref, establish_peering_request_config
    )
    if async_:
      return establish_op
    operation_ref = resources.REGISTRY.ParseRelativeName(
        establish_op.name, collection=constants.OPERATIONS_COLLECTION
    )
    return self.WaitForOperation(operation_ref)

  def SyncReplication(self, replication_ref, async_):
    """Sync a Cloud NetApp Volume Replication.

    Args:
      replication_ref: the reference to the Replication.
      async_: bool, if False, wait for the operation to complete.

    Returns:
      an Operation or Volume message.
    """
    sync_op = self._adapter.SyncReplication(replication_ref)
    if async_:
      return sync_op
    operation_ref = resources.REGISTRY.ParseRelativeName(
        sync_op.name, collection=constants.OPERATIONS_COLLECTION
    )
    return self.WaitForOperation(operation_ref)


class ReplicationsAdapter(object):
  """Adapter for the Cloud NetApp Files API Replication resource."""

  def __init__(self):
    self.release_track = base.ReleaseTrack.GA
    self.client = netapp_api_util.GetClientInstance(
        release_track=self.release_track
    )
    self.messages = netapp_api_util.GetMessagesModule(
        release_track=self.release_track
    )

  def ParseDestinationVolumeParameters(
      self, replication, destination_volume_parameters
  ):
    """Parses Destination Volume Parameters for Replication into a config.

    Args:
      replication: The Cloud Netapp Volumes Replication object.
      destination_volume_parameters: The Destination Volume Parameters message
        object.

    Returns:
      Replication message populated with Destination Volume Parameters values.
    """
    if not destination_volume_parameters:
      return
    parameters = self.messages.DestinationVolumeParameters()
    for key, val in destination_volume_parameters.items():
      if key == 'storage_pool':
        parameters.storagePool = val
      elif key == 'volume_id':
        parameters.volumeId = val
      elif key == 'share_name':
        parameters.shareName = val
      elif key == 'description':
        parameters.description = val
      elif key == 'tiering_policy':
        parameters.tieringPolicy = self.messages.TieringPolicy(
            tierAction=val['tier-action'],
            coolingThresholdDays=val['cooling-threshold-days'],
        )
      else:
        log.warning('The attribute {} is not recognized.'.format(key))
    replication.destinationVolumeParameters = parameters

  def UpdateReplication(self, replication_ref, replication_config, update_mask):
    """Send a Patch request for the Cloud NetApp Volume Replication."""
    update_request = (
        self.messages.NetappProjectsLocationsVolumesReplicationsPatchRequest(
            replication=replication_config,
            name=replication_ref.RelativeName(),
            updateMask=update_mask,
        )
    )
    update_op = self.client.projects_locations_volumes_replications.Patch(
        update_request
    )
    return update_op

  def ParseUpdatedReplicationConfig(
      self, replication_config, description=None, labels=None,
      replication_schedule=None, cluster_location=None
  ):
    """Parse update information into an updated Replication message."""
    if description is not None:
      replication_config.description = description
    if labels is not None:
      replication_config.labels = labels
    if replication_schedule is not None:
      replication_config.replicationSchedule = replication_schedule
    if cluster_location is not None:
      replication_config.clusterLocation = cluster_location
    return replication_config

  def ResumeReplication(self, replication_ref):
    """Send a resume request for the Cloud NetApp Volume Replication."""
    resume_request = (
        self.messages.NetappProjectsLocationsVolumesReplicationsResumeRequest(
            name=replication_ref.RelativeName(),
        )
    )
    return self.client.projects_locations_volumes_replications.Resume(
        resume_request
    )

  def ReverseReplication(self, replication_ref):
    """Send a reverse request for the Cloud NetApp Volume Replication."""
    reverse_request = (
        self.messages.NetappProjectsLocationsVolumesReplicationsReverseDirectionRequest(
            name=replication_ref.RelativeName(),
        )
    )
    return self.client.projects_locations_volumes_replications.ReverseDirection(
        reverse_request
    )

  def StopReplication(self, replication_ref, force):
    """Send a stop request for the Cloud NetApp Volume Replication."""
    stop_request = (
        self.messages.NetappProjectsLocationsVolumesReplicationsStopRequest(
            name=replication_ref.RelativeName(),
            stopReplicationRequest=self.messages.StopReplicationRequest(
                force=force
            ),
        )
    )
    return self.client.projects_locations_volumes_replications.Stop(
        stop_request
    )

  def ParseEstablishPeeringRequestConfig(
      self,
      peer_cluster_name,
      peer_svm_name,
      peer_volume_name,
      peer_ip_addresses,
  ):
    """Parse establish peering request for the Cloud NetApp Volume Replication."""
    establish_peering_request = self.messages.EstablishPeeringRequest(
        peerClusterName=peer_cluster_name,
        peerSvmName=peer_svm_name,
        peerVolumeName=peer_volume_name,
        peerIpAddresses=peer_ip_addresses if peer_ip_addresses else [],
    )
    return establish_peering_request

  def EstablishPeering(self, replication_ref, establish_peering_request_config):
    """Send a establish peering request for the Cloud NetApp Volume Replication."""
    establish_peering_request = (
        self.messages.NetappProjectsLocationsVolumesReplicationsEstablishPeeringRequest(
            name=replication_ref.RelativeName(),
            establishPeeringRequest=establish_peering_request_config,
        )
    )
    return self.client.projects_locations_volumes_replications.EstablishPeering(
        establish_peering_request
    )

  def SyncReplication(self, replication_ref):
    """Send a sync request for the Cloud NetApp Volume Replication."""
    sync_request = (
        self.messages.NetappProjectsLocationsVolumesReplicationsSyncRequest(
            name=replication_ref.RelativeName(),
        )
    )
    return self.client.projects_locations_volumes_replications.Sync(
        sync_request
    )


class BetaReplicationsAdapter(ReplicationsAdapter):
  """Adapter for the Beta Cloud NetApp Files API Replication resource."""

  def __init__(self):
    super(BetaReplicationsAdapter, self).__init__()
    self.release_track = base.ReleaseTrack.BETA
    self.client = netapp_api_util.GetClientInstance(
        release_track=self.release_track
    )
    self.messages = netapp_api_util.GetMessagesModule(
        release_track=self.release_track
    )


class AlphaReplicationsAdapter(BetaReplicationsAdapter):
  """Adapter for the Alpha Cloud NetApp Files API Replication resource."""

  def __init__(self):
    super(AlphaReplicationsAdapter, self).__init__()
    self.release_track = base.ReleaseTrack.ALPHA
    self.client = netapp_api_util.GetClientInstance(
        release_track=self.release_track
    )
    self.messages = netapp_api_util.GetMessagesModule(
        release_track=self.release_track
    )



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The gcloud run instances dev group."""

from googlecloudsdk.calliope import base


@base.UniverseCompatible
class Dev(base.Group):
  """Commands for development on Cloud Run Instances.

  This set of commands can be used for development on Cloud Run Instances.

  To start new sync session, use `{parent_command} sync`.
  """

  detailed_help = {
      'EXAMPLES': (
          """
          To start a new sync session, run:

            $ {command} sync my-instance --source=.
      """
      ),
  }



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities used by gcloud functions local development."""


import json
import re
import textwrap
from googlecloudsdk.core import exceptions as core_exceptions
from googlecloudsdk.core import execution_utils
from googlecloudsdk.core.util import files

import six

_INSTALLATION_GUIDE = textwrap.dedent("""\
    You must install Docker and Pack to run this command.
    To install Docker and Pack, please follow this guide:
    https://cloud.google.com/functions/1stgendocs/running/functions-emulator""")
_DOCKER = files.FindExecutableOnPath('docker')
_PACK = files.FindExecutableOnPath('pack')
_APPENGINE_BUILDER = 'gcr.io/serverless-runtimes/google-{}-full/builder/{}'
_V1_BUILDER = 'gcr.io/buildpacks/builder:v1'
_GOOGLE_22_BUILDER = 'gcr.io/buildpacks/builder:google-22'
_RUNTIME_MINVERSION_UBUNTU_22 = {'python': 310, 'nodejs': 18, 'go': 116,
                                 'java': 17, 'php': 82, 'ruby': 32,
                                 'dotnet': 6}


class MissingExecutablesException(core_exceptions.Error):
  """Executables for local development are not found."""


class ContainerNotFoundException(core_exceptions.Error):
  """Docker container is not found."""


class DockerExecutionException(core_exceptions.Error):
  """Docker executable exited with non-zero code."""


class PackExecutionException(core_exceptions.Error):
  """Pack executable exited with non-zero code."""


def ValidateDependencies():
  if _DOCKER is None or _PACK is None:
    raise MissingExecutablesException(_INSTALLATION_GUIDE)


def RunPack(name, builder, runtime, entry_point, path, build_env_vars):
  """Runs Pack Build with the command built from arguments of the command parser.

  Args:
    name: Name of the image build.
    builder: Name of the builder by the flag.
    runtime: Runtime specified by flag.
    entry_point: Entry point of the function specified by flag.
    path: Source of the zip file.
    build_env_vars: Build environment variables.
  Raises:
    PackExecutionException: if the exit code of the execution is non-zero.
  """
  pack_cmd = [_PACK, 'build', '--builder']

  # Always use language-specific builder when builder not provided.
  if not builder:
    [language, version] = re.findall(r'(\D+|\d+)', runtime)
    if int(version) >= _RUNTIME_MINVERSION_UBUNTU_22[language]:
      builder = (_GOOGLE_22_BUILDER if language == 'dotnet'
                 else _APPENGINE_BUILDER.format(22, language))
    else:
      builder = (_V1_BUILDER if language == 'dotnet'
                 else _APPENGINE_BUILDER.format(18, language))
  pack_cmd.append(builder)

  if build_env_vars:
    _AddEnvVars(pack_cmd, build_env_vars)

  pack_cmd.extend(['--env', 'GOOGLE_FUNCTION_TARGET=' + entry_point])
  pack_cmd.extend(['--path', path])
  pack_cmd.extend(['-q', name])
  status = execution_utils.Exec(pack_cmd, no_exit=True)
  if status:
    raise PackExecutionException(
        status, 'Pack failed to build the container image.')


def RunDockerContainer(name, port, env_vars, labels):
  """Runs the Docker container (detached mode) with specified port and name.

  If the name already exists, it will be removed.

  Args:
    name: The name of the container to run.
    port: The port for the container to run on.
    env_vars: The container environment variables.
    labels: Docker labels to store flags and environment variables.

  Raises:
    DockerExecutionException: if the exit code of the execution is non-zero.
  """
  if ContainerExists(name):
    RemoveDockerContainer(name)
  docker_cmd = [_DOCKER, 'run', '-d']
  docker_cmd.extend(['-p', six.text_type(port) + ':8080'])
  if env_vars:
    _AddEnvVars(docker_cmd, env_vars)
  for k, v in labels.items():
    docker_cmd.extend(['--label', '{}={}'.format(k, json.dumps(v))])
  docker_cmd.extend(['--name', name, name])
  status = execution_utils.Exec(docker_cmd, no_exit=True)
  if status:
    raise DockerExecutionException(
        status, 'Docker failed to run container ' + name)


def RemoveDockerContainer(name):
  """Removes the Docker container with specified name.

  Args:
    name: The name of the Docker container to delete.

  Raises:
    DockerExecutionException: if the exit code of the execution is non-zero.
  """
  delete_cmd = [_DOCKER, 'rm', '-f', name]
  status = execution_utils.Exec(delete_cmd, no_exit=True)
  if status:
    raise DockerExecutionException(
        status, 'Docker failed to execute: failed to remove container ' + name)


def ContainerExists(name):
  """Returns True if the Docker container with specified name exists.

  Args:
    name: The name of the Docker container.

  Returns:
    bool: True if the container exists, False otherwise.

  Raises:
    DockerExecutionException: if the exit code of the execution is non-zero.
  """
  list_cmd = [_DOCKER, 'ps', '-q', '-f', 'name=' + name]
  out = []
  capture_out = lambda stdout: out.append(stdout.strip())
  status = execution_utils.Exec(list_cmd, out_func=capture_out, no_exit=True)
  if status:
    raise DockerExecutionException(
        status, 'Docker failed to execute: failed to list container ' + name)
  return bool(out[0])


def FindContainerPort(name):
  """Returns the port of the Docker container with specified name.

  Args:
    name: The name of the Docker container.

  Returns:
    str: The port number of the Docker container.

  Raises:
    DockerExecutionException: if the exit code of the execution is non-zero
    or if the port of the container does not exist.
  """
  mapping = """{{range $p, $conf := .NetworkSettings.Ports}}\
      {{(index $conf 0).HostPort}}{{end}}"""
  find_port = [_DOCKER, 'inspect', '--format=' + mapping, name]
  out = []
  capture_out = lambda stdout: out.append(stdout.strip())
  status = execution_utils.Exec(find_port, out_func=capture_out, no_exit=True)
  if status:
    raise DockerExecutionException(
        status, 'Docker failed to execute: failed to find port for ' + name)
  return out[0]


def GetDockerContainerLabels(name):
  """Returns the labels of the Docker container with specified name.

  Args:
    name: The name of the Docker container.

  Returns:
    dict: The labels for the docker container in json format.

  Raises:
    DockerExecutionException: if the exit code of the execution is non-zero
    or if the port of the container does not exist.
  """
  if not ContainerExists(name):
    return {}
  find_labels = [_DOCKER, 'inspect', '--format={{json .Config.Labels}}', name]
  out = []
  capture_out = lambda stdout: out.append(stdout.strip())
  status = execution_utils.Exec(find_labels, out_func=capture_out, no_exit=True)
  if status:
    raise DockerExecutionException(
        status, 'Docker failed to execute: failed to labels for ' + name)
  return json.loads(out[0])


def _AddEnvVars(cmd_args, env_vars):
  for key, value in env_vars.items():
    cmd_args.extend(['--env', key + '=' + value])



"""Generated client library for metastore version v1beta."""
# NOTE: This file is autogenerated and should not be edited by hand.

from apitools.base.py import base_api
from googlecloudsdk.generated_clients.apis.metastore.v1beta import metastore_v1beta_messages as messages


class MetastoreV1beta(base_api.BaseApiClient):
  """Generated client library for service metastore version v1beta."""

  MESSAGES_MODULE = messages
  BASE_URL = 'https://metastore.googleapis.com/'
  MTLS_BASE_URL = 'https://metastore.mtls.googleapis.com/'

  _PACKAGE = 'metastore'
  _SCOPES = ['https://www.googleapis.com/auth/cloud-platform']
  _VERSION = 'v1beta'
  _CLIENT_ID = 'CLIENT_ID'
  _CLIENT_SECRET = 'CLIENT_SECRET'
  _USER_AGENT = 'google-cloud-sdk'
  _CLIENT_CLASS_NAME = 'MetastoreV1beta'
  _URL_VERSION = 'v1beta'
  _API_KEY = None

  def __init__(self, url='', credentials=None,
               get_credentials=True, http=None, model=None,
               log_request=False, log_response=False,
               credentials_args=None, default_global_params=None,
               additional_http_headers=None, response_encoding=None):
    """Create a new metastore handle."""
    url = url or self.BASE_URL
    super(MetastoreV1beta, self).__init__(
        url, credentials=credentials,
        get_credentials=get_credentials, http=http, model=model,
        log_request=log_request, log_response=log_response,
        credentials_args=credentials_args,
        default_global_params=default_global_params,
        additional_http_headers=additional_http_headers,
        response_encoding=response_encoding)
    self.projects_locations_federations = self.ProjectsLocationsFederationsService(self)
    self.projects_locations_operations = self.ProjectsLocationsOperationsService(self)
    self.projects_locations_services_backups = self.ProjectsLocationsServicesBackupsService(self)
    self.projects_locations_services_databases_tables = self.ProjectsLocationsServicesDatabasesTablesService(self)
    self.projects_locations_services_databases = self.ProjectsLocationsServicesDatabasesService(self)
    self.projects_locations_services_metadataImports = self.ProjectsLocationsServicesMetadataImportsService(self)
    self.projects_locations_services_migrationExecutions = self.ProjectsLocationsServicesMigrationExecutionsService(self)
    self.projects_locations_services = self.ProjectsLocationsServicesService(self)
    self.projects_locations = self.ProjectsLocationsService(self)
    self.projects = self.ProjectsService(self)

  class ProjectsLocationsFederationsService(base_api.BaseApiService):
    """Service class for the projects_locations_federations resource."""

    _NAME = 'projects_locations_federations'

    def __init__(self, client):
      super(MetastoreV1beta.ProjectsLocationsFederationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a metastore federation in a project and location.

      Args:
        request: (MetastoreProjectsLocationsFederationsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/federations',
        http_method='POST',
        method_id='metastore.projects.locations.federations.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['federationId', 'requestId'],
        relative_path='v1beta/{+parent}/federations',
        request_field='federation',
        request_type_name='MetastoreProjectsLocationsFederationsCreateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a single federation.

      Args:
        request: (MetastoreProjectsLocationsFederationsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/federations/{federationsId}',
        http_method='DELETE',
        method_id='metastore.projects.locations.federations.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['requestId'],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='MetastoreProjectsLocationsFederationsDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets the details of a single federation.

      Args:
        request: (MetastoreProjectsLocationsFederationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Federation) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/federations/{federationsId}',
        http_method='GET',
        method_id='metastore.projects.locations.federations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='MetastoreProjectsLocationsFederationsGetRequest',
        response_type_name='Federation',
        supports_download=False,
    )

    def GetIamPolicy(self, request, global_params=None):
      r"""Gets the access control policy for a resource. Returns an empty policy if the resource exists and does not have a policy set.

      Args:
        request: (MetastoreProjectsLocationsFederationsGetIamPolicyRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Policy) The response message.
      """
      config = self.GetMethodConfig('GetIamPolicy')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetIamPolicy.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/federations/{federationsId}:getIamPolicy',
        http_method='GET',
        method_id='metastore.projects.locations.federations.getIamPolicy',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=['options_requestedPolicyVersion'],
        relative_path='v1beta/{+resource}:getIamPolicy',
        request_field='',
        request_type_name='MetastoreProjectsLocationsFederationsGetIamPolicyRequest',
        response_type_name='Policy',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists federations in a project and location.

      Args:
        request: (MetastoreProjectsLocationsFederationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListFederationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/federations',
        http_method='GET',
        method_id='metastore.projects.locations.federations.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1beta/{+parent}/federations',
        request_field='',
        request_type_name='MetastoreProjectsLocationsFederationsListRequest',
        response_type_name='ListFederationsResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates the fields of a federation.

      Args:
        request: (MetastoreProjectsLocationsFederationsPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/federations/{federationsId}',
        http_method='PATCH',
        method_id='metastore.projects.locations.federations.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['requestId', 'updateMask'],
        relative_path='v1beta/{+name}',
        request_field='federation',
        request_type_name='MetastoreProjectsLocationsFederationsPatchRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def SetIamPolicy(self, request, global_params=None):
      r"""Sets the access control policy on the specified resource. Replaces any existing policy.Can return NOT_FOUND, INVALID_ARGUMENT, and PERMISSION_DENIED errors.

      Args:
        request: (MetastoreProjectsLocationsFederationsSetIamPolicyRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Policy) The response message.
      """
      config = self.GetMethodConfig('SetIamPolicy')
      return self._RunMethod(
          config, request, global_params=global_params)

    SetIamPolicy.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/federations/{federationsId}:setIamPolicy',
        http_method='POST',
        method_id='metastore.projects.locations.federations.setIamPolicy',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=[],
        relative_path='v1beta/{+resource}:setIamPolicy',
        request_field='setIamPolicyRequest',
        request_type_name='MetastoreProjectsLocationsFederationsSetIamPolicyRequest',
        response_type_name='Policy',
        supports_download=False,
    )

    def TestIamPermissions(self, request, global_params=None):
      r"""Returns permissions that a caller has on the specified resource. If the resource does not exist, this will return an empty set of permissions, not a NOT_FOUND error.Note: This operation is designed to be used for building permission-aware UIs and command-line tools, not for authorization checking. This operation may "fail open" without warning.

      Args:
        request: (MetastoreProjectsLocationsFederationsTestIamPermissionsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (TestIamPermissionsResponse) The response message.
      """
      config = self.GetMethodConfig('TestIamPermissions')
      return self._RunMethod(
          config, request, global_params=global_params)

    TestIamPermissions.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/federations/{federationsId}:testIamPermissions',
        http_method='POST',
        method_id='metastore.projects.locations.federations.testIamPermissions',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=[],
        relative_path='v1beta/{+resource}:testIamPermissions',
        request_field='testIamPermissionsRequest',
        request_type_name='MetastoreProjectsLocationsFederationsTestIamPermissionsRequest',
        response_type_name='TestIamPermissionsResponse',
        supports_download=False,
    )

  class ProjectsLocationsOperationsService(base_api.BaseApiService):
    """Service class for the projects_locations_operations resource."""

    _NAME = 'projects_locations_operations'

    def __init__(self, client):
      super(MetastoreV1beta.ProjectsLocationsOperationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Cancel(self, request, global_params=None):
      r"""Starts asynchronous cancellation on a long-running operation. The server makes a best effort to cancel the operation, but success is not guaranteed. If the server doesn't support this method, it returns google.rpc.Code.UNIMPLEMENTED. Clients can use Operations.GetOperation or other methods to check whether the cancellation succeeded or whether the operation completed despite cancellation. On successful cancellation, the operation is not deleted; instead, it becomes an operation with an Operation.error value with a google.rpc.Status.code of 1, corresponding to Code.CANCELLED.

      Args:
        request: (MetastoreProjectsLocationsOperationsCancelRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Cancel')
      return self._RunMethod(
          config, request, global_params=global_params)

    Cancel.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}:cancel',
        http_method='POST',
        method_id='metastore.projects.locations.operations.cancel',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta/{+name}:cancel',
        request_field='cancelOperationRequest',
        request_type_name='MetastoreProjectsLocationsOperationsCancelRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a long-running operation. This method indicates that the client is no longer interested in the operation result. It does not cancel the operation. If the server doesn't support this method, it returns google.rpc.Code.UNIMPLEMENTED.

      Args:
        request: (MetastoreProjectsLocationsOperationsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}',
        http_method='DELETE',
        method_id='metastore.projects.locations.operations.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='MetastoreProjectsLocationsOperationsDeleteRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets the latest state of a long-running operation. Clients can use this method to poll the operation result at intervals as recommended by the API service.

      Args:
        request: (MetastoreProjectsLocationsOperationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}',
        http_method='GET',
        method_id='metastore.projects.locations.operations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='MetastoreProjectsLocationsOperationsGetRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists operations that match the specified filter in the request. If the server doesn't support this method, it returns UNIMPLEMENTED.

      Args:
        request: (MetastoreProjectsLocationsOperationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListOperationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/operations',
        http_method='GET',
        method_id='metastore.projects.locations.operations.list',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['filter', 'pageSize', 'pageToken', 'returnPartialSuccess'],
        relative_path='v1beta/{+name}/operations',
        request_field='',
        request_type_name='MetastoreProjectsLocationsOperationsListRequest',
        response_type_name='ListOperationsResponse',
        supports_download=False,
    )

  class ProjectsLocationsServicesBackupsService(base_api.BaseApiService):
    """Service class for the projects_locations_services_backups resource."""

    _NAME = 'projects_locations_services_backups'

    def __init__(self, client):
      super(MetastoreV1beta.ProjectsLocationsServicesBackupsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a new backup in a given project and location.

      Args:
        request: (MetastoreProjectsLocationsServicesBackupsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/services/{servicesId}/backups',
        http_method='POST',
        method_id='metastore.projects.locations.services.backups.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['backupId', 'requestId'],
        relative_path='v1beta/{+parent}/backups',
        request_field='backup',
        request_type_name='MetastoreProjectsLocationsServicesBackupsCreateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a single backup.

      Args:
        request: (MetastoreProjectsLocationsServicesBackupsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/services/{servicesId}/backups/{backupsId}',
        http_method='DELETE',
        method_id='metastore.projects.locations.services.backups.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['requestId'],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='MetastoreProjectsLocationsServicesBackupsDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets details of a single backup.

      Args:
        request: (MetastoreProjectsLocationsServicesBackupsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Backup) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/services/{servicesId}/backups/{backupsId}',
        http_method='GET',
        method_id='metastore.projects.locations.services.backups.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='MetastoreProjectsLocationsServicesBackupsGetRequest',
        response_type_name='Backup',
        supports_download=False,
    )

    def GetIamPolicy(self, request, global_params=None):
      r"""Gets the access control policy for a resource. Returns an empty policy if the resource exists and does not have a policy set.

      Args:
        request: (MetastoreProjectsLocationsServicesBackupsGetIamPolicyRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Policy) The response message.
      """
      config = self.GetMethodConfig('GetIamPolicy')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetIamPolicy.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/services/{servicesId}/backups/{backupsId}:getIamPolicy',
        http_method='GET',
        method_id='metastore.projects.locations.services.backups.getIamPolicy',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=['options_requestedPolicyVersion'],
        relative_path='v1beta/{+resource}:getIamPolicy',
        request_field='',
        request_type_name='MetastoreProjectsLocationsServicesBackupsGetIamPolicyRequest',
        response_type_name='Policy',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists backups in a service.

      Args:
        request: (MetastoreProjectsLocationsServicesBackupsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListBackupsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/services/{servicesId}/backups',
        http_method='GET',
        method_id='metastore.projects.locations.services.backups.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1beta/{+parent}/backups',
        request_field='',
        request_type_name='MetastoreProjectsLocationsServicesBackupsListRequest',
        response_type_name='ListBackupsResponse',
        supports_download=False,
    )

    def SetIamPolicy(self, request, global_params=None):
      r"""Sets the access control policy on the specified resource. Replaces any existing policy.Can return NOT_FOUND, INVALID_ARGUMENT, and PERMISSION_DENIED errors.

      Args:
        request: (MetastoreProjectsLocationsServicesBackupsSetIamPolicyRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Policy) The response message.
      """
      config = self.GetMethodConfig('SetIamPolicy')
      return self._RunMethod(
          config, request, global_params=global_params)

    SetIamPolicy.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/services/{servicesId}/backups/{backupsId}:setIamPolicy',
        http_method='POST',
        method_id='metastore.projects.locations.services.backups.setIamPolicy',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=[],
        relative_path='v1beta/{+resource}:setIamPolicy',
        request_field='setIamPolicyRequest',
        request_type_name='MetastoreProjectsLocationsServicesBackupsSetIamPolicyRequest',
        response_type_name='Policy',
        supports_download=False,
    )

    def TestIamPermissions(self, request, global_params=None):
      r"""Returns permissions that a caller has on the specified resource. If the resource does not exist, this will return an empty set of permissions, not a NOT_FOUND error.Note: This operation is designed to be used for building permission-aware UIs and command-line tools, not for authorization checking. This operation may "fail open" without warning.

      Args:
        request: (MetastoreProjectsLocationsServicesBackupsTestIamPermissionsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (TestIamPermissionsResponse) The response message.
      """
      config = self.GetMethodConfig('TestIamPermissions')
      return self._RunMethod(
          config, request, global_params=global_params)

    TestIamPermissions.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/services/{servicesId}/backups/{backupsId}:testIamPermissions',
        http_method='POST',
        method_id='metastore.projects.locations.services.backups.testIamPermissions',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=[],
        relative_path='v1beta/{+resource}:testIamPermissions',
        request_field='testIamPermissionsRequest',
        request_type_name='MetastoreProjectsLocationsServicesBackupsTestIamPermissionsRequest',
        response_type_name='TestIamPermissionsResponse',
        supports_download=False,
    )

  class ProjectsLocationsServicesDatabasesTablesService(base_api.BaseApiService):
    """Service class for the projects_locations_services_databases_tables resource."""

    _NAME = 'projects_locations_services_databases_tables'

    def __init__(self, client):
      super(MetastoreV1beta.ProjectsLocationsServicesDatabasesTablesService, self).__init__(client)
      self._upload_configs = {
          }

    def GetIamPolicy(self, request, global_params=None):
      r"""Gets the access control policy for a resource. Returns an empty policy if the resource exists and does not have a policy set.

      Args:
        request: (MetastoreProjectsLocationsServicesDatabasesTablesGetIamPolicyRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Policy) The response message.
      """
      config = self.GetMethodConfig('GetIamPolicy')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetIamPolicy.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/services/{servicesId}/databases/{databasesId}/tables/{tablesId}:getIamPolicy',
        http_method='GET',
        method_id='metastore.projects.locations.services.databases.tables.getIamPolicy',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=['options_requestedPolicyVersion'],
        relative_path='v1beta/{+resource}:getIamPolicy',
        request_field='',
        request_type_name='MetastoreProjectsLocationsServicesDatabasesTablesGetIamPolicyRequest',
        response_type_name='Policy',
        supports_download=False,
    )

    def SetIamPolicy(self, request, global_params=None):
      r"""Sets the access control policy on the specified resource. Replaces any existing policy.Can return NOT_FOUND, INVALID_ARGUMENT, and PERMISSION_DENIED errors.

      Args:
        request: (MetastoreProjectsLocationsServicesDatabasesTablesSetIamPolicyRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Policy) The response message.
      """
      config = self.GetMethodConfig('SetIamPolicy')
      return self._RunMethod(
          config, request, global_params=global_params)

    SetIamPolicy.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/services/{servicesId}/databases/{databasesId}/tables/{tablesId}:setIamPolicy',
        http_method='POST',
        method_id='metastore.projects.locations.services.databases.tables.setIamPolicy',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=[],
        relative_path='v1beta/{+resource}:setIamPolicy',
        request_field='setIamPolicyRequest',
        request_type_name='MetastoreProjectsLocationsServicesDatabasesTablesSetIamPolicyRequest',
        response_type_name='Policy',
        supports_download=False,
    )

    def TestIamPermissions(self, request, global_params=None):
      r"""Returns permissions that a caller has on the specified resource. If the resource does not exist, this will return an empty set of permissions, not a NOT_FOUND error.Note: This operation is designed to be used for building permission-aware UIs and command-line tools, not for authorization checking. This operation may "fail open" without warning.

      Args:
        request: (MetastoreProjectsLocationsServicesDatabasesTablesTestIamPermissionsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (TestIamPermissionsResponse) The response message.
      """
      config = self.GetMethodConfig('TestIamPermissions')
      return self._RunMethod(
          config, request, global_params=global_params)

    TestIamPermissions.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/services/{servicesId}/databases/{databasesId}/tables/{tablesId}:testIamPermissions',
        http_method='POST',
        method_id='metastore.projects.locations.services.databases.tables.testIamPermissions',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=[],
        relative_path='v1beta/{+resource}:testIamPermissions',
        request_field='testIamPermissionsRequest',
        request_type_name='MetastoreProjectsLocationsServicesDatabasesTablesTestIamPermissionsRequest',
        response_type_name='TestIamPermissionsResponse',
        supports_download=False,
    )

  class ProjectsLocationsServicesDatabasesService(base_api.BaseApiService):
    """Service class for the projects_locations_services_databases resource."""

    _NAME = 'projects_locations_services_databases'

    def __init__(self, client):
      super(MetastoreV1beta.ProjectsLocationsServicesDatabasesService, self).__init__(client)
      self._upload_configs = {
          }

    def GetIamPolicy(self, request, global_params=None):
      r"""Gets the access control policy for a resource. Returns an empty policy if the resource exists and does not have a policy set.

      Args:
        request: (MetastoreProjectsLocationsServicesDatabasesGetIamPolicyRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Policy) The response message.
      """
      config = self.GetMethodConfig('GetIamPolicy')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetIamPolicy.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/services/{servicesId}/databases/{databasesId}:getIamPolicy',
        http_method='GET',
        method_id='metastore.projects.locations.services.databases.getIamPolicy',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=['options_requestedPolicyVersion'],
        relative_path='v1beta/{+resource}:getIamPolicy',
        request_field='',
        request_type_name='MetastoreProjectsLocationsServicesDatabasesGetIamPolicyRequest',
        response_type_name='Policy',
        supports_download=False,
    )

    def SetIamPolicy(self, request, global_params=None):
      r"""Sets the access control policy on the specified resource. Replaces any existing policy.Can return NOT_FOUND, INVALID_ARGUMENT, and PERMISSION_DENIED errors.

      Args:
        request: (MetastoreProjectsLocationsServicesDatabasesSetIamPolicyRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Policy) The response message.
      """
      config = self.GetMethodConfig('SetIamPolicy')
      return self._RunMethod(
          config, request, global_params=global_params)

    SetIamPolicy.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/services/{servicesId}/databases/{databasesId}:setIamPolicy',
        http_method='POST',
        method_id='metastore.projects.locations.services.databases.setIamPolicy',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=[],
        relative_path='v1beta/{+resource}:setIamPolicy',
        request_field='setIamPolicyRequest',
        request_type_name='MetastoreProjectsLocationsServicesDatabasesSetIamPolicyRequest',
        response_type_name='Policy',
        supports_download=False,
    )

    def TestIamPermissions(self, request, global_params=None):
      r"""Returns permissions that a caller has on the specified resource. If the resource does not exist, this will return an empty set of permissions, not a NOT_FOUND error.Note: This operation is designed to be used for building permission-aware UIs and command-line tools, not for authorization checking. This operation may "fail open" without warning.

      Args:
        request: (MetastoreProjectsLocationsServicesDatabasesTestIamPermissionsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (TestIamPermissionsResponse) The response message.
      """
      config = self.GetMethodConfig('TestIamPermissions')
      return self._RunMethod(
          config, request, global_params=global_params)

    TestIamPermissions.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/services/{servicesId}/databases/{databasesId}:testIamPermissions',
        http_method='POST',
        method_id='metastore.projects.locations.services.databases.testIamPermissions',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=[],
        relative_path='v1beta/{+resource}:testIamPermissions',
        request_field='testIamPermissionsRequest',
        request_type_name='MetastoreProjectsLocationsServicesDatabasesTestIamPermissionsRequest',
        response_type_name='TestIamPermissionsResponse',
        supports_download=False,
    )

  class ProjectsLocationsServicesMetadataImportsService(base_api.BaseApiService):
    """Service class for the projects_locations_services_metadataImports resource."""

    _NAME = 'projects_locations_services_metadataImports'

    def __init__(self, client):
      super(MetastoreV1beta.ProjectsLocationsServicesMetadataImportsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a new MetadataImport in a given project and location.

      Args:
        request: (MetastoreProjectsLocationsServicesMetadataImportsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/services/{servicesId}/metadataImports',
        http_method='POST',
        method_id='metastore.projects.locations.services.metadataImports.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['metadataImportId', 'requestId'],
        relative_path='v1beta/{+parent}/metadataImports',
        request_field='metadataImport',
        request_type_name='MetastoreProjectsLocationsServicesMetadataImportsCreateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets details of a single import.

      Args:
        request: (MetastoreProjectsLocationsServicesMetadataImportsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (MetadataImport) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/services/{servicesId}/metadataImports/{metadataImportsId}',
        http_method='GET',
        method_id='metastore.projects.locations.services.metadataImports.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='MetastoreProjectsLocationsServicesMetadataImportsGetRequest',
        response_type_name='MetadataImport',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists imports in a service.

      Args:
        request: (MetastoreProjectsLocationsServicesMetadataImportsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListMetadataImportsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/services/{servicesId}/metadataImports',
        http_method='GET',
        method_id='metastore.projects.locations.services.metadataImports.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1beta/{+parent}/metadataImports',
        request_field='',
        request_type_name='MetastoreProjectsLocationsServicesMetadataImportsListRequest',
        response_type_name='ListMetadataImportsResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates a single import. Only the description field of MetadataImport is supported to be updated.

      Args:
        request: (MetastoreProjectsLocationsServicesMetadataImportsPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/services/{servicesId}/metadataImports/{metadataImportsId}',
        http_method='PATCH',
        method_id='metastore.projects.locations.services.metadataImports.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['requestId', 'updateMask'],
        relative_path='v1beta/{+name}',
        request_field='metadataImport',
        request_type_name='MetastoreProjectsLocationsServicesMetadataImportsPatchRequest',
        response_type_name='Operation',
        supports_download=False,
    )

  class ProjectsLocationsServicesMigrationExecutionsService(base_api.BaseApiService):
    """Service class for the projects_locations_services_migrationExecutions resource."""

    _NAME = 'projects_locations_services_migrationExecutions'

    def __init__(self, client):
      super(MetastoreV1beta.ProjectsLocationsServicesMigrationExecutionsService, self).__init__(client)
      self._upload_configs = {
          }

    def Delete(self, request, global_params=None):
      r"""Deletes a single migration execution.

      Args:
        request: (MetastoreProjectsLocationsServicesMigrationExecutionsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/services/{servicesId}/migrationExecutions/{migrationExecutionsId}',
        http_method='DELETE',
        method_id='metastore.projects.locations.services.migrationExecutions.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['requestId'],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='MetastoreProjectsLocationsServicesMigrationExecutionsDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets details of a single migration execution.

      Args:
        request: (MetastoreProjectsLocationsServicesMigrationExecutionsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (MigrationExecution) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/services/{servicesId}/migrationExecutions/{migrationExecutionsId}',
        http_method='GET',
        method_id='metastore.projects.locations.services.migrationExecutions.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='MetastoreProjectsLocationsServicesMigrationExecutionsGetRequest',
        response_type_name='MigrationExecution',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists migration executions on a service.

      Args:
        request: (MetastoreProjectsLocationsServicesMigrationExecutionsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListMigrationExecutionsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/services/{servicesId}/migrationExecutions',
        http_method='GET',
        method_id='metastore.projects.locations.services.migrationExecutions.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1beta/{+parent}/migrationExecutions',
        request_field='',
        request_type_name='MetastoreProjectsLocationsServicesMigrationExecutionsListRequest',
        response_type_name='ListMigrationExecutionsResponse',
        supports_download=False,
    )

  class ProjectsLocationsServicesService(base_api.BaseApiService):
    """Service class for the projects_locations_services resource."""

    _NAME = 'projects_locations_services'

    def __init__(self, client):
      super(MetastoreV1beta.ProjectsLocationsServicesService, self).__init__(client)
      self._upload_configs = {
          }

    def AlterLocation(self, request, global_params=None):
      r"""Alter metadata resource location. The metadata resource can be a database, table, or partition. This functionality only updates the parent directory for the respective metadata resource and does not transfer any existing data to the new location.

      Args:
        request: (MetastoreProjectsLocationsServicesAlterLocationRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('AlterLocation')
      return self._RunMethod(
          config, request, global_params=global_params)

    AlterLocation.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/services/{servicesId}:alterLocation',
        http_method='POST',
        method_id='metastore.projects.locations.services.alterLocation',
        ordered_params=['service'],
        path_params=['service'],
        query_params=[],
        relative_path='v1beta/{+service}:alterLocation',
        request_field='alterMetadataResourceLocationRequest',
        request_type_name='MetastoreProjectsLocationsServicesAlterLocationRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def AlterTableProperties(self, request, global_params=None):
      r"""Alter metadata table properties.

      Args:
        request: (MetastoreProjectsLocationsServicesAlterTablePropertiesRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('AlterTableProperties')
      return self._RunMethod(
          config, request, global_params=global_params)

    AlterTableProperties.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/services/{servicesId}:alterTableProperties',
        http_method='POST',
        method_id='metastore.projects.locations.services.alterTableProperties',
        ordered_params=['service'],
        path_params=['service'],
        query_params=[],
        relative_path='v1beta/{+service}:alterTableProperties',
        request_field='alterTablePropertiesRequest',
        request_type_name='MetastoreProjectsLocationsServicesAlterTablePropertiesRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def CancelMigration(self, request, global_params=None):
      r"""Cancels the ongoing Managed Migration process.

      Args:
        request: (MetastoreProjectsLocationsServicesCancelMigrationRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('CancelMigration')
      return self._RunMethod(
          config, request, global_params=global_params)

    CancelMigration.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/services/{servicesId}:cancelMigration',
        http_method='POST',
        method_id='metastore.projects.locations.services.cancelMigration',
        ordered_params=['service'],
        path_params=['service'],
        query_params=[],
        relative_path='v1beta/{+service}:cancelMigration',
        request_field='cancelMigrationRequest',
        request_type_name='MetastoreProjectsLocationsServicesCancelMigrationRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def CompleteMigration(self, request, global_params=None):
      r"""Completes the managed migration process. The Dataproc Metastore service will switch to using its own backend database after successful migration.

      Args:
        request: (MetastoreProjectsLocationsServicesCompleteMigrationRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('CompleteMigration')
      return self._RunMethod(
          config, request, global_params=global_params)

    CompleteMigration.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/services/{servicesId}:completeMigration',
        http_method='POST',
        method_id='metastore.projects.locations.services.completeMigration',
        ordered_params=['service'],
        path_params=['service'],
        query_params=[],
        relative_path='v1beta/{+service}:completeMigration',
        request_field='completeMigrationRequest',
        request_type_name='MetastoreProjectsLocationsServicesCompleteMigrationRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Create(self, request, global_params=None):
      r"""Creates a metastore service in a project and location.

      Args:
        request: (MetastoreProjectsLocationsServicesCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/services',
        http_method='POST',
        method_id='metastore.projects.locations.services.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['requestId', 'serviceId'],
        relative_path='v1beta/{+parent}/services',
        request_field='service',
        request_type_name='MetastoreProjectsLocationsServicesCreateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a single service.

      Args:
        request: (MetastoreProjectsLocationsServicesDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/services/{servicesId}',
        http_method='DELETE',
        method_id='metastore.projects.locations.services.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['requestId'],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='MetastoreProjectsLocationsServicesDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def ExportMetadata(self, request, global_params=None):
      r"""Exports metadata from a service.

      Args:
        request: (MetastoreProjectsLocationsServicesExportMetadataRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('ExportMetadata')
      return self._RunMethod(
          config, request, global_params=global_params)

    ExportMetadata.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/services/{servicesId}:exportMetadata',
        http_method='POST',
        method_id='metastore.projects.locations.services.exportMetadata',
        ordered_params=['service'],
        path_params=['service'],
        query_params=[],
        relative_path='v1beta/{+service}:exportMetadata',
        request_field='exportMetadataRequest',
        request_type_name='MetastoreProjectsLocationsServicesExportMetadataRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets the details of a single service.

      Args:
        request: (MetastoreProjectsLocationsServicesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Service) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/services/{servicesId}',
        http_method='GET',
        method_id='metastore.projects.locations.services.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='MetastoreProjectsLocationsServicesGetRequest',
        response_type_name='Service',
        supports_download=False,
    )

    def GetIamPolicy(self, request, global_params=None):
      r"""Gets the access control policy for a resource. Returns an empty policy if the resource exists and does not have a policy set.

      Args:
        request: (MetastoreProjectsLocationsServicesGetIamPolicyRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Policy) The response message.
      """
      config = self.GetMethodConfig('GetIamPolicy')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetIamPolicy.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/services/{servicesId}:getIamPolicy',
        http_method='GET',
        method_id='metastore.projects.locations.services.getIamPolicy',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=['options_requestedPolicyVersion'],
        relative_path='v1beta/{+resource}:getIamPolicy',
        request_field='',
        request_type_name='MetastoreProjectsLocationsServicesGetIamPolicyRequest',
        response_type_name='Policy',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists services in a project and location.

      Args:
        request: (MetastoreProjectsLocationsServicesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListServicesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/services',
        http_method='GET',
        method_id='metastore.projects.locations.services.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1beta/{+parent}/services',
        request_field='',
        request_type_name='MetastoreProjectsLocationsServicesListRequest',
        response_type_name='ListServicesResponse',
        supports_download=False,
    )

    def MoveTableToDatabase(self, request, global_params=None):
      r"""Move a table to another database.

      Args:
        request: (MetastoreProjectsLocationsServicesMoveTableToDatabaseRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('MoveTableToDatabase')
      return self._RunMethod(
          config, request, global_params=global_params)

    MoveTableToDatabase.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/services/{servicesId}:moveTableToDatabase',
        http_method='POST',
        method_id='metastore.projects.locations.services.moveTableToDatabase',
        ordered_params=['service'],
        path_params=['service'],
        query_params=[],
        relative_path='v1beta/{+service}:moveTableToDatabase',
        request_field='moveTableToDatabaseRequest',
        request_type_name='MetastoreProjectsLocationsServicesMoveTableToDatabaseRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates the parameters of a single service.

      Args:
        request: (MetastoreProjectsLocationsServicesPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/services/{servicesId}',
        http_method='PATCH',
        method_id='metastore.projects.locations.services.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['requestId', 'updateMask'],
        relative_path='v1beta/{+name}',
        request_field='service',
        request_type_name='MetastoreProjectsLocationsServicesPatchRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def QueryMetadata(self, request, global_params=None):
      r"""Query Dataproc Metastore metadata.

      Args:
        request: (MetastoreProjectsLocationsServicesQueryMetadataRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('QueryMetadata')
      return self._RunMethod(
          config, request, global_params=global_params)

    QueryMetadata.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/services/{servicesId}:queryMetadata',
        http_method='POST',
        method_id='metastore.projects.locations.services.queryMetadata',
        ordered_params=['service'],
        path_params=['service'],
        query_params=[],
        relative_path='v1beta/{+service}:queryMetadata',
        request_field='queryMetadataRequest',
        request_type_name='MetastoreProjectsLocationsServicesQueryMetadataRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def RemoveIamPolicy(self, request, global_params=None):
      r"""Removes the attached IAM policies for a resource.

      Args:
        request: (MetastoreProjectsLocationsServicesRemoveIamPolicyRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (RemoveIamPolicyResponse) The response message.
      """
      config = self.GetMethodConfig('RemoveIamPolicy')
      return self._RunMethod(
          config, request, global_params=global_params)

    RemoveIamPolicy.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/services/{servicesId}/{servicesId1}:removeIamPolicy',
        http_method='POST',
        method_id='metastore.projects.locations.services.removeIamPolicy',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=[],
        relative_path='v1beta/{+resource}:removeIamPolicy',
        request_field='removeIamPolicyRequest',
        request_type_name='MetastoreProjectsLocationsServicesRemoveIamPolicyRequest',
        response_type_name='RemoveIamPolicyResponse',
        supports_download=False,
    )

    def Restore(self, request, global_params=None):
      r"""Restores a service from a backup.

      Args:
        request: (MetastoreProjectsLocationsServicesRestoreRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Restore')
      return self._RunMethod(
          config, request, global_params=global_params)

    Restore.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/services/{servicesId}:restore',
        http_method='POST',
        method_id='metastore.projects.locations.services.restore',
        ordered_params=['service'],
        path_params=['service'],
        query_params=[],
        relative_path='v1beta/{+service}:restore',
        request_field='restoreServiceRequest',
        request_type_name='MetastoreProjectsLocationsServicesRestoreRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def SetIamPolicy(self, request, global_params=None):
      r"""Sets the access control policy on the specified resource. Replaces any existing policy.Can return NOT_FOUND, INVALID_ARGUMENT, and PERMISSION_DENIED errors.

      Args:
        request: (MetastoreProjectsLocationsServicesSetIamPolicyRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Policy) The response message.
      """
      config = self.GetMethodConfig('SetIamPolicy')
      return self._RunMethod(
          config, request, global_params=global_params)

    SetIamPolicy.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/services/{servicesId}:setIamPolicy',
        http_method='POST',
        method_id='metastore.projects.locations.services.setIamPolicy',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=[],
        relative_path='v1beta/{+resource}:setIamPolicy',
        request_field='setIamPolicyRequest',
        request_type_name='MetastoreProjectsLocationsServicesSetIamPolicyRequest',
        response_type_name='Policy',
        supports_download=False,
    )

    def StartMigration(self, request, global_params=None):
      r"""Starts the Managed Migration process.

      Args:
        request: (MetastoreProjectsLocationsServicesStartMigrationRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('StartMigration')
      return self._RunMethod(
          config, request, global_params=global_params)

    StartMigration.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/services/{servicesId}:startMigration',
        http_method='POST',
        method_id='metastore.projects.locations.services.startMigration',
        ordered_params=['service'],
        path_params=['service'],
        query_params=[],
        relative_path='v1beta/{+service}:startMigration',
        request_field='startMigrationRequest',
        request_type_name='MetastoreProjectsLocationsServicesStartMigrationRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def TestIamPermissions(self, request, global_params=None):
      r"""Returns permissions that a caller has on the specified resource. If the resource does not exist, this will return an empty set of permissions, not a NOT_FOUND error.Note: This operation is designed to be used for building permission-aware UIs and command-line tools, not for authorization checking. This operation may "fail open" without warning.

      Args:
        request: (MetastoreProjectsLocationsServicesTestIamPermissionsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (TestIamPermissionsResponse) The response message.
      """
      config = self.GetMethodConfig('TestIamPermissions')
      return self._RunMethod(
          config, request, global_params=global_params)

    TestIamPermissions.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/services/{servicesId}:testIamPermissions',
        http_method='POST',
        method_id='metastore.projects.locations.services.testIamPermissions',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=[],
        relative_path='v1beta/{+resource}:testIamPermissions',
        request_field='testIamPermissionsRequest',
        request_type_name='MetastoreProjectsLocationsServicesTestIamPermissionsRequest',
        response_type_name='TestIamPermissionsResponse',
        supports_download=False,
    )

  class ProjectsLocationsService(base_api.BaseApiService):
    """Service class for the projects_locations resource."""

    _NAME = 'projects_locations'

    def __init__(self, client):
      super(MetastoreV1beta.ProjectsLocationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Gets information about a location.

      Args:
        request: (MetastoreProjectsLocationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Location) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}',
        http_method='GET',
        method_id='metastore.projects.locations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='MetastoreProjectsLocationsGetRequest',
        response_type_name='Location',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists information about the supported locations for this service.This method lists locations based on the resource scope provided in the ListLocationsRequest.name field: Global locations: If name is empty, the method lists the public locations available to all projects. Project-specific locations: If name follows the format projects/{project}, the method lists locations visible to that specific project. This includes public, private, or other project-specific locations enabled for the project.For gRPC and client library implementations, the resource name is passed as the name field. For direct service calls, the resource name is incorporated into the request path based on the specific service implementation and version.

      Args:
        request: (MetastoreProjectsLocationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListLocationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations',
        http_method='GET',
        method_id='metastore.projects.locations.list',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['extraLocationTypes', 'filter', 'pageSize', 'pageToken'],
        relative_path='v1beta/{+name}/locations',
        request_field='',
        request_type_name='MetastoreProjectsLocationsListRequest',
        response_type_name='ListLocationsResponse',
        supports_download=False,
    )

  class ProjectsService(base_api.BaseApiService):
    """Service class for the projects resource."""

    _NAME = 'projects'

    def __init__(self, client):
      super(MetastoreV1beta.ProjectsService, self).__init__(client)
      self._upload_configs = {
          }



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Shared util methods common to BQExports commands."""


import re

from googlecloudsdk.command_lib.scc import errors
from googlecloudsdk.command_lib.scc import util


def ValidateAndGetBigQueryExportV1Name(args):
  """Returns relative resource name for a v1 B2igQuery export.

  Validates on regexes for args containing full names or short names with
  resources. Localization is supported by the
  ValidateAndGetBigQueryExportV2Name method.

  Args:
    args: an argparse object that should contain .BIG_QUERY_EXPORT, optionally 1
      of .organization, .folder, .project

  Examples:

  args with BIG_QUERY_EXPORT="organizations/123/bigQueryExports/config1"
  returns the BIG_QUERY_EXPORT

  args with BIG_QUERY_EXPORT="config1" and projects="projects/123" returns
  projects/123/bigQueryExports/config1
  """
  bq_export_name = args.BIG_QUERY_EXPORT

  long_name_format = re.compile(
      "(organizations|projects|folders)/.*/bigQueryExports/[a-z]([a-z0-9-]{0,61}[a-z0-9])?$"
  ).match(bq_export_name)
  short_name_format = re.compile("^[a-z]([a-z0-9-]{0,61}[a-z0-9])?$").match(
      bq_export_name
  )

  if not long_name_format and not short_name_format:
    if "/" in bq_export_name:
      raise errors.InvalidSCCInputError(
          "BigQuery export must match the full resource name, or "
          "`--organization=`, `--folder=` or `--project=` must be provided."
      )
    else:
      raise errors.InvalidSCCInputError(
          "BigQuery export id does not match the pattern "
          "'^[a-z]([a-z0-9-]{0,61}[a-z0-9])?$'."
      )

  if long_name_format:
    return bq_export_name

  if short_name_format:
    parent = util.GetParentFromNamedArguments(args)
    if parent is None:
      raise errors.InvalidSCCInputError(
          "BigQuery export must match the full resource name, or "
          "`--organization=`, `--folder=` or `--project=` must be provided."
      )
    else:
      return (
          util.GetParentFromNamedArguments(args)
          + "/bigQueryExports/"
          + bq_export_name
      )


def ValidateAndGetBigQueryExportV2Name(args):
  """Returns relative resource name for a v2 Big Query export.

  Validates on regexes for args containing full names with locations or short
  names with resources.

  Args:
    args: an argparse object that should contain .BIG_QUERY_EXPORT, optionally 1
      of .organization, .folder, .project; and optionally .location

  Examples:

  args with BIG_QUERY_EXPORT="organizations/123/bigQueryExports/config1"
  and location="locations/us" returns
  organizations/123/locations/us/bigQueryExports/config1

  args with
  BIG_QUERY_EXPORT="folders/123/locations/us/bigQueryExports/config1"
  and returns folders/123/locations/us/bigQueryExports/config1

  args with BIG_QUERY_EXPORT="config1", projects="projects/123", and
  locations="us" returns projects/123/bigQueryExports/config1
  """

  id_pattern = re.compile("^[a-z]([a-z0-9-]{0,61}[a-z0-9])?$")
  nonregionalized_resource_pattern = re.compile(
      "(organizations|projects|folders)/.+/bigQueryExports/[a-z]([a-z0-9-]{0,61}[a-z0-9])?$"
  )

  regionalized_resource_pattern = re.compile(
      "(organizations|projects|folders)/.+/locations/.+/bigQueryExports/[a-z]([a-z0-9-]{0,61}[a-z0-9])?$"
  )
  bq_export_id = args.BIG_QUERY_EXPORT
  location = util.ValidateAndGetLocation(args, "v2")

  # id-only pattern (short name): compose the full name
  if id_pattern.match(bq_export_id):
    parent = util.GetParentFromNamedArguments(args)
    if parent is None:
      raise errors.InvalidSCCInputError(
          "BigQuery export must match the full resource name, or "
          "`--organization=`, `--folder=` or `--project=` must be provided."
      )
    return f"{parent}/locations/{location}/bigQueryExports/{bq_export_id}"

  # v2=style regionalized patterns
  if regionalized_resource_pattern.match(bq_export_id):
    return bq_export_id

  # v1-style nonregionalized patterns are acceptable
  if nonregionalized_resource_pattern.match(bq_export_id):
    # Handle config id as full resource name
    [parent_segment, id_segment] = bq_export_id.split("/bigQueryExports/")
    return f"{parent_segment}/locations/{location}/bigQueryExports/{id_segment}"

  raise errors.InvalidSCCInputError(
      "BigQuery export must match"
      " (organizations|projects|folders)/.+/bigQueryExports/[a-z]([a-z0-9-]{0,61}[a-z0-9])?$"
      " (organizations|projects|folders)/.+/locations/.+/bigQueryExports/[a-z]([a-z0-9-]{0,61}[a-z0-9])?$"
      " or [a-zA-Z0-9-_]{1,128}$."
  )




# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities for the parsing output for cloud build v2 API."""
import re


def ParseName(pattern, primitive_type):
  """Parses the name of a pipelineRun/taskRun.

  Args:
    pattern:
      "projects/{project}/locations/{location}/pipelineRuns/{pipeline_run}"
      "projects/{project}/locations/{location}/taskRuns/{task_run}"
    primitive_type: string

  Returns:
    name: string
  """
  if primitive_type == "pipelinerun":
    match = re.match(
        r"projects/([^/]+)/locations/([^/]+)/pipelineRuns/([^/]+)", pattern
    )
    if match:
      return match.group(3)
  elif primitive_type == "taskrun":
    match = re.match(
        r"projects/([^/]+)/locations/([^/]+)/taskRuns/([^/]+)", pattern
    )
    if match:
      return match.group(3)



# -*- coding: utf-8 -*- #
# Copyright 2014 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for adding access configs to virtual machine instances."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute import constants
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute.instances import flags


DETAILED_HELP = {
    'DESCRIPTION': """
*{command}* is used to create access configurations for network
interfaces of Compute Engine virtual machines. This allows you
to assign a public, external IP to a virtual machine.
""",
    'EXAMPLES': """
To assign an public, externally accessible IP to a virtual machine named
``example-instance'' in zone ``us-central1-a'', run:

  $ {command} example-instance --zone=us-central1-a

To assign the specific, reserved public IP address ``123.456.789.123''
to the virtual machine, run:

  $ {command} example-instance --zone=us-central1-a --address=123.456.789.123
""",
}


def _Args(parser, support_public_dns):
  """Register parser args common to all tracks."""

  parser.add_argument(
      '--access-config-name',
      default=constants.DEFAULT_ACCESS_CONFIG_NAME,
      help="""\
      Specifies the name of the new access configuration. ``{0}''
      is used as the default if this flag is not provided. Since ONE_TO_ONE_NAT
      is currently the only access-config type, it is not recommended that you
      change this value.
      """.format(constants.DEFAULT_ACCESS_CONFIG_NAME))

  parser.add_argument(
      '--address',
      action=arg_parsers.StoreOnceAction,
      help="""\
      Specifies the external IP address of the new access
      configuration. If this is not specified, then the service will
      choose an available ephemeral IP address. If an explicit IP
      address is given, then that IP address must be reserved by the
      project and not be in use by another resource.
      """)

  flags.AddNetworkInterfaceArgs(parser)
  flags.AddPublicPtrArgs(parser, instance=False)
  if support_public_dns:
    flags.AddPublicDnsArgs(parser, instance=False)
  flags.AddNetworkTierArgs(parser, instance=False)
  flags.INSTANCE_ARG.AddArgument(parser)


@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.PREVIEW)
class AddAccessConfigInstances(base.SilentCommand):
  """Create a Compute Engine virtual machine access configuration."""

  _support_public_dns = False

  @classmethod
  def Args(cls, parser):
    _Args(parser, support_public_dns=cls._support_public_dns)

  def Run(self, args):
    """Invokes request necessary for adding an access config."""
    flags.ValidateNetworkTierArgs(args)

    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client

    instance_ref = flags.INSTANCE_ARG.ResolveAsResource(
        args,
        holder.resources,
        scope_lister=flags.GetInstanceZoneScopeLister(client))

    access_config = client.messages.AccessConfig(
        name=args.access_config_name,
        natIP=args.address,
        type=client.messages.AccessConfig.TypeValueValuesEnum.ONE_TO_ONE_NAT)

    if self._support_public_dns:
      flags.ValidatePublicDnsFlags(args)
      if args.no_public_dns is True:
        access_config.setPublicDns = False
      elif args.public_dns is True:
        access_config.setPublicDns = True

    flags.ValidatePublicPtrFlags(args)
    if args.no_public_ptr is True:
      access_config.setPublicPtr = False
    elif args.public_ptr is True:
      access_config.setPublicPtr = True

    if (args.no_public_ptr_domain is not True and
        args.public_ptr_domain is not None):
      access_config.publicPtrDomainName = args.public_ptr_domain

    network_tier = getattr(args, 'network_tier', None)
    if network_tier is not None:
      access_config.networkTier = (client.messages.AccessConfig.
                                   NetworkTierValueValuesEnum(network_tier))

    request = client.messages.ComputeInstancesAddAccessConfigRequest(
        accessConfig=access_config,
        instance=instance_ref.Name(),
        networkInterface=args.network_interface,
        project=instance_ref.project,
        zone=instance_ref.zone)

    return client.MakeRequests([(client.apitools_client.instances,
                                 'AddAccessConfig', request)])


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class AddAccessConfigInstancesBeta(AddAccessConfigInstances):
  """Create a Compute Engine virtual machine access configuration."""

  _support_public_dns = False


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class AddAccessConfigInstancesAlpha(AddAccessConfigInstances):
  """Create a Compute Engine virtual machine access configuration."""

  _support_public_dns = True

AddAccessConfigInstances.detailed_help = DETAILED_HELP



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Common flags for fleet packages commands."""

from googlecloudsdk.calliope.concepts import concepts
from googlecloudsdk.calliope.concepts import deps
from googlecloudsdk.core import properties
from googlecloudsdk.core import resources


def GetProject(args):
  return args.project or properties.VALUES.core.project.Get(required=True)


def GetLocation(args):
  return args.location or properties.VALUES.config_delivery.location.Get(
      required=True
  )


def AddLessFlag(parser):
  parser.add_argument(
      '--less',
      required=False,
      default=False,
      action='store_true',
      help='Show less verbose output.',
  )


def AddNameFlag(parser):
  parser.add_argument('name', help='Resource name.')


def AddReleaseFlag(parser):
  parser.add_argument(
      'release', help='Release identifier, either a version or tag.'
  )


def AddFleetPackageFlag(parser):
  parser.add_argument(
      '--fleet-package',
      required=True,
      help='Parent Fleet Package of the Rollout.',
  )


def AddSourceFlag(parser):
  parser.add_argument(
      '--source',
      required=True,
      help='Source file containing Fleet Package configuration.',
  )


def AddLocationFlag(parser):
  parser.add_argument(
      '--location', required=False, help='Google Cloud zone or region.'
  )


def AddDescriptionFlag(parser):
  parser.add_argument(
      '--description', required=False, help='Resource description.'
  )


def AddResourceBundleFlag(parser):
  parser.add_argument(
      '--resource-bundle', required=True, help='Resource Bundle name.'
  )


def AddSkipCreatingVariantResourcesFlag(parser):
  parser.add_argument(
      '--skip-creating-variant-resources',
      required=False,
      help='Whether to opt-in to the alternate variant upload flow.',
      default=False,
      hidden=True,
      action='store_true',
  )


def AddForceDeleteFlag(parser, resource_name):
  parser.add_argument(
      '--force',
      required=False,
      default=False,
      action='store_true',
      help=(
          'If true, force deletion of any child resources. Otherwise,'
          f' attempting to delete a {resource_name} with children will fail.'
      ),
  )


def AddLifecycleFlag(parser):
  parser.add_argument(
      '--lifecycle', required=False, help='Lifecycle of the Release.'
  )


def ProjectAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='project',
      help_text='Project ID for the {resource}.',
      fallthroughs=[deps.PropertyFallthrough(properties.VALUES.core.project)],
  )


def LocationAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='location',
      help_text='Google Cloud zone or region for the {resource}.',
      fallthroughs=[
          deps.PropertyFallthrough(properties.VALUES.config_delivery.location)
      ],
  )


def FleetPackageAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='fleet-package', help_text='Fleet Package name.'
  )


def RolloutAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='rollout', help_text='Rollout name.'
  )


def GetFleetPackageResourceSpec():
  return concepts.ResourceSpec(
      'configdelivery.projects.locations.fleetPackages',
      resource_name='fleet-package',
      fleetPackagesId=FleetPackageAttributeConfig(),
      locationsId=LocationAttributeConfig(),
      projectsId=ProjectAttributeConfig(),
  )


def GetRolloutResourceSpec():
  return concepts.ResourceSpec(
      'configdelivery.projects.locations.fleetPackages.rollouts',
      resource_name='rollout',
      rolloutsId=RolloutAttributeConfig(),
      fleetPackagesId=FleetPackageAttributeConfig(),
      locationsId=LocationAttributeConfig(),
      projectsId=ProjectAttributeConfig(),
  )


def AddUriFlags(parser, collection, api_version):
  """Adds `--uri` flag to the parser object for list commands.

  Args:
    parser: The argparse parser.
    collection: str, The resource collection name.
    api_version: str, The API version to use.
  """

  def _GetResourceUri(resource):
    resource_relative_name = resources.REGISTRY.ParseRelativeName(
        resource.name, collection=collection, api_version=api_version
    )
    return resource_relative_name.SelfLink()

  parser.display_info.AddUriFunc(_GetResourceUri)



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to generate a new Audit Report."""


from apitools.base.py import exceptions as apitools_exceptions
from googlecloudsdk.api_lib.audit_manager import audit_reports
from googlecloudsdk.api_lib.audit_manager import constants
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.audit_manager import exception_utils
from googlecloudsdk.command_lib.audit_manager import flags
from googlecloudsdk.core import exceptions as core_exceptions
from googlecloudsdk.core import properties


_DETAILED_HELP = {
    'DESCRIPTION': 'Generate a new Audit Report.',
    'EXAMPLES': """ \
        To generate an Audit Report in the `us-central1` region,
        for a project with ID `123` for compliance framework `fedramp_moderate` in `odf` format and store it in `gs://testbucketauditmanager` bucket, run:

          $ {command} --project=123 --location=us-central1 --compliance-framework=fedramp_moderate --report-format=odf --gcs-uri=gs://testbucketauditmanager
        """,
}


@base.DefaultUniverseOnly
@base.ReleaseTracks(base.ReleaseTrack.GA)
class Generate(base.CreateCommand):
  """Generate Audit Report."""

  detailed_help = _DETAILED_HELP
  api_version = constants.ApiVersion.V1

  @staticmethod
  def Args(parser):
    flags.AddProjectOrFolderFlags(parser, 'for which to generate audit report')
    flags.AddLocationFlag(parser, 'the report should be generated')
    flags.AddComplianceFrameworkFlag(parser)
    flags.AddReportFormatFlag(parser)
    flags.AddDestinationFlags(parser)
    parser.display_info.AddFormat(properties.VALUES.core.default_format.Get())

  def Run(self, args):
    """Run the generate command."""
    is_parent_folder = args.folder is not None

    scope = (
        'folders/{folder}'.format(folder=args.folder)
        if is_parent_folder
        else 'projects/{project}'.format(project=args.project)
    )

    scope += '/locations/{location}'.format(location=args.location)

    client = audit_reports.AuditReportsClient(api_version=self.api_version)

    try:
      return client.Generate(
          scope,
          args.compliance_framework,
          report_format=args.report_format,
          gcs_uri=args.gcs_uri,
          is_parent_folder=is_parent_folder,
      )

    except apitools_exceptions.HttpError as error:
      exc = exception_utils.AuditManagerError(error)

      if exc.has_error_info(exception_utils.ERROR_REASON_NOT_ENROLLED):
        exc.suggested_command_purpose = 'enroll the resource'
        exc.suggested_command = (
            f'{flags.GetCommandPrefix(args.command_path)} enrollments add'
            f' {flags.GetProjectOrFolderParam(args)} {flags.GetLocationParam(args)}'
            f' {flags.GetEligibleGcsBucketParam(args)}'
        )
      elif exc.has_error_info(exception_utils.ERROR_REASON_PERMISSION_DENIED):
        role = 'roles/auditmanager.auditor'
        user = properties.VALUES.core.account.Get()
        exc.suggested_command_purpose = 'grant permission'
        command_prefix = (
            'gcloud resource-manager folders add-iam-policy-binding'
            if is_parent_folder
            else 'gcloud projects add-iam-policy-binding'
        )
        exc.suggested_command = (
            f'{command_prefix}'
            f' {args.folder if is_parent_folder else args.project}'
            f' --member=user:{user} --role {role}'
        )

      core_exceptions.reraise(exc)


@base.DefaultUniverseOnly
@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class GenerateAlpha(Generate):
  """Generate Audit Report."""

  api_version = constants.ApiVersion.V1_ALPHA



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Shared resource flags for Network Security commands."""


from googlecloudsdk.calliope import base
from googlecloudsdk.calliope.concepts import concepts
from googlecloudsdk.calliope.concepts import deps
from googlecloudsdk.command_lib.util.concepts import concept_parsers
from googlecloudsdk.command_lib.util.concepts import presentation_specs


def _ServerTlsPolicyAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='server_tls_policy',
      help_text='ID of the server TLS policy for {resource}.',
  )


def _LocationAttributeConfig(region_fallthrough):
  fallthroughs = []
  if region_fallthrough:
    fallthroughs.append(deps.ArgFallthrough('--region'))
  fallthroughs.append(
      deps.Fallthrough(
          lambda: 'global', 'default value of location is [global]'
      )
  )
  return concepts.ResourceParameterAttributeConfig(
      name='location',
      help_text='The Cloud location for the {resource}.',
      fallthroughs=fallthroughs,
  )


def _GetServerTlsPolicyResourceSpec(region_fallthrough):
  return concepts.ResourceSpec(
      'networksecurity.projects.locations.serverTlsPolicies',
      resource_name='server_tls_policy',
      serverTlsPoliciesId=_ServerTlsPolicyAttributeConfig(),
      locationsId=_LocationAttributeConfig(region_fallthrough),
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      disable_auto_completers=False,
  )


def _GetServerTlsPolicyResourcePresentationSpec(
    flag,
    noun,
    verb,
    required=False,
    plural=False,
    group=None,
    region_fallthrough=False,
):
  """Returns ResourcePresentationSpec for server TLS policy resource.

  Args:
    flag: str, the flag name.
    noun: str, the resource.
    verb: str, the verb to describe the resource, such as 'to update'.
    required: bool, if False, means that map ID is optional.
    plural: bool.
    group: args group.
    region_fallthrough: bool, True if the command has a region flag that should
      be used as a fallthrough for the server TLS policy location.

  Returns:
    presentation_specs.ResourcePresentationSpec.
  """
  # 'location' flag is overridden to make it invisible in the documentation.
  flag_overrides = {'location': ''}
  return presentation_specs.ResourcePresentationSpec(
      flag,
      _GetServerTlsPolicyResourceSpec(region_fallthrough),
      '{} {}.'.format(noun, verb),
      required=required,
      plural=plural,
      group=group,
      flag_name_overrides=flag_overrides,
  )


def GetServerTlsPolicyResourceArg(
    verb,
    noun='The server TLS policy',
    name='server-tls-policy',
    required=False,
    plural=False,
    group=None,
    region_fallthrough=False,
):
  """Creates a resource argument for a Server TLS policy.

  Args:
    verb: str, the verb to describe the resource, such as 'to update'.
    noun: str, the resource; default: 'The server TLS policy'.
    name: str, the name of the flag.
    required: bool, if True the flag is required.
    plural: bool, if True the flag is a list.
    group: args group.
    region_fallthrough: bool, True if the command has a region flag that should
      be used as a fallthrough for the server TLS policy location.

  Returns:
    ServerTlsPolicyResourceArg: ConceptParser, holder for Server TLS policy
    argument.
  """
  return concept_parsers.ConceptParser([
      _GetServerTlsPolicyResourcePresentationSpec(
          '--' + name, noun, verb, required, plural, group, region_fallthrough
      ),
  ])


def GetClearServerTLSPolicyForHttpsProxy(name='clear-server-tls-policy'):
  """Returns the flag for clearing the Server TLS policy.

  Args:
    name: str, the name of the flag; default: 'clear-server-tls-policy'.
  """

  return base.Argument(
      '--' + name,
      action='store_true',
      default=False,
      required=False,
      help="""\
      Removes any attached Server TLS policy.
      """,
  )



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to generate service agents for Workload Identity."""

from apitools.base.py import encoding
from googlecloudsdk.api_lib.util import waiter
from googlecloudsdk.api_lib.workloadidentity import service_agents
from googlecloudsdk.api_lib.workloadidentity import util as api_util
from googlecloudsdk.calliope import base
from googlecloudsdk.core import exceptions
from googlecloudsdk.core import log


@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA, base.ReleaseTrack.GA
)
@base.UniverseCompatible
class Generate(base.CreateCommand):
  """Generate service agents for a service producer.

  It can be generated in a project, folder, or organization and location.

  `{command}` generates service agents for a given service producer in a
  specific project, folder, or organization and location.

  ## EXAMPLES

  To generate service agents for `bigquery.googleapis.com` in the `global`
  location for the project `123456`, run:

    $ {command} --service="bigquery.googleapis.com" --location="global"
    --project="123456"

  To generate service agents for `bigquery.googleapis.com` in the `global`
  location for the folder `123456`, run:

    $ {command} --service="bigquery.googleapis.com" --location="global"
    --folder="123456"

  To generate service agents for `bigquery.googleapis.com` in the `global`
  location for the organization `123456`, run:

    $ {command} --service="bigquery.googleapis.com" --location="global"
    --organization="123456"
  """

  @staticmethod
  def Args(parser):
    """Register flags for this command."""
    parser.add_argument(
        '--service',
        required=True,
        help=(
            'The service producer to generate service agents for (e.g.'
            ' bigquery.googleapis.com).'
        ),
    )

    parser.add_argument(
        '--location',
        required=False,
        default='global',
        help=(
            'The location for which to generate the service agents. Defaults to'
            ' global.'
        ),
    )

    scope_group = parser.add_mutually_exclusive_group(required=True)
    scope_group.add_argument(
        '--project',
        help='The project number to generate service agents for.',
    )
    scope_group.add_argument(
        '--folder',
        help='The folder number to generate service agents for.',
    )
    scope_group.add_argument(
        '--organization',
        help='The organization number to generate service agents for.',
    )

  def Run(self, args):
    """Run the generating command."""
    location = args.location
    service = args.service
    scope_type = None
    scope_id = None

    if args.project:
      scope_type = 'projects'
      scope_id = args.project
    elif args.folder:
      scope_type = 'folders'
      scope_id = args.folder
    elif args.organization:
      scope_type = 'organizations'
      scope_id = args.organization
    # No else is needed here because the argument group is required.

    container = f'{scope_type}/{scope_id}'
    parent = f'{container}/locations/{location}/serviceProducers/{service}'
    msg = (
        f'Generating service agents for {service} in {location} '
        f'under {container}'
    )

    op = service_agents.GenerateServiceAgents(parent, self.ReleaseTrack())

    if not op:
      raise exceptions.Error(
          'Service agents generation did not complete successfully.'
      )

    client = api_util.GetClientInstance(self.ReleaseTrack())
    poller = waiter.CloudOperationPollerNoResources(
        client.projects_locations_operations, get_name_func=lambda x: x
    )
    result = waiter.WaitFor(poller, op.name, msg)
    response_dict = encoding.MessageToPyValue(result) if result else {}
    service_agents_list = response_dict.get('serviceAgents', [])

    log.status.Print(
        f'\nProvisioned service agents for {service} under {container}:\n'
    )
    for sa_dict in service_agents_list:
      for key, value in sa_dict.items():
        log.status.Print(f'{key}: {value}')
      log.status.Print('----')

    return result
